# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.enums.types import account_budget_proposal_type
from google.ads.googleads.v22.enums.types import account_budget_status
from google.ads.googleads.v22.enums.types import (
    spending_limit_type as gage_spending_limit_type,
)
from google.ads.googleads.v22.enums.types import time_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "AccountBudget",
    },
)


class AccountBudget(proto.Message):
    r"""An account-level budget. It contains information about the budget
    itself, as well as the most recently approved changes to the budget
    and proposed changes that are pending approval. The proposed changes
    that are pending approval, if any, are found in 'pending_proposal'.
    Effective details about the budget are found in fields prefixed
    'approved\_', 'adjusted\_' and those without a prefix. Since some
    effective details may differ from what the user had originally
    requested (for example, spending limit), these differences are
    juxtaposed through 'proposed\_', 'approved\_', and possibly
    'adjusted\_' fields.

    This resource is mutated using AccountBudgetProposal and cannot be
    mutated directly. A budget may have at most one pending proposal at
    any given time. It is read through pending_proposal.

    Once approved, a budget may be subject to adjustments, such as
    credit adjustments. Adjustments create differences between the
    'approved' and 'adjusted' fields, which would otherwise be
    identical.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the account-level budget.
            AccountBudget resource names have the form:

            ``customers/{customer_id}/accountBudgets/{account_budget_id}``
        id (int):
            Output only. The ID of the account-level
            budget.

            This field is a member of `oneof`_ ``_id``.
        billing_setup (str):
            Output only. The resource name of the billing setup
            associated with this account-level budget. BillingSetup
            resource names have the form:

            ``customers/{customer_id}/billingSetups/{billing_setup_id}``

            This field is a member of `oneof`_ ``_billing_setup``.
        status (google.ads.googleads.v22.enums.types.AccountBudgetStatusEnum.AccountBudgetStatus):
            Output only. The status of this account-level
            budget.
        name (str):
            Output only. The name of the account-level
            budget.

            This field is a member of `oneof`_ ``_name``.
        proposed_start_date_time (str):
            Output only. The proposed start time of the
            account-level budget in yyyy-MM-dd HH:mm:ss
            format.  If a start time type of NOW was
            proposed, this is the time of request.

            This field is a member of `oneof`_ ``_proposed_start_date_time``.
        approved_start_date_time (str):
            Output only. The approved start time of the
            account-level budget in yyyy-MM-dd HH:mm:ss
            format.

            For example, if a new budget is approved after
            the proposed start time, the approved start time
            is the time of approval.

            This field is a member of `oneof`_ ``_approved_start_date_time``.
        total_adjustments_micros (int):
            Output only. The total adjustments amount.

            An example of an adjustment is courtesy credits.
        amount_served_micros (int):
            Output only. The value of Ads that have been served, in
            micros.

            This includes overdelivery costs, in which case a credit
            might be automatically applied to the budget (see
            total_adjustments_micros).
        purchase_order_number (str):
            Output only. A purchase order number is a
            value that helps users reference this budget in
            their monthly invoices.

            This field is a member of `oneof`_ ``_purchase_order_number``.
        notes (str):
            Output only. Notes associated with the
            budget.

            This field is a member of `oneof`_ ``_notes``.
        pending_proposal (google.ads.googleads.v22.resources.types.AccountBudget.PendingAccountBudgetProposal):
            Output only. The pending proposal to modify
            this budget, if applicable.
        proposed_end_date_time (str):
            Output only. The proposed end time in
            yyyy-MM-dd HH:mm:ss format.

            This field is a member of `oneof`_ ``proposed_end_time``.
        proposed_end_time_type (google.ads.googleads.v22.enums.types.TimeTypeEnum.TimeType):
            Output only. The proposed end time as a
            well-defined type, for example, FOREVER.

            This field is a member of `oneof`_ ``proposed_end_time``.
        approved_end_date_time (str):
            Output only. The approved end time in
            yyyy-MM-dd HH:mm:ss format.

            This field is a member of `oneof`_ ``approved_end_time``.
        approved_end_time_type (google.ads.googleads.v22.enums.types.TimeTypeEnum.TimeType):
            Output only. The approved end time as a
            well-defined type, for example, FOREVER.

            This field is a member of `oneof`_ ``approved_end_time``.
        proposed_spending_limit_micros (int):
            Output only. The proposed spending limit in
            micros.  One million is equivalent to one unit.

            This field is a member of `oneof`_ ``proposed_spending_limit``.
        proposed_spending_limit_type (google.ads.googleads.v22.enums.types.SpendingLimitTypeEnum.SpendingLimitType):
            Output only. The proposed spending limit as a
            well-defined type, for example, INFINITE.

            This field is a member of `oneof`_ ``proposed_spending_limit``.
        approved_spending_limit_micros (int):
            Output only. The approved spending limit in
            micros.  One million is equivalent to one unit.
            This will only be populated if the proposed
            spending limit is finite, and will always be
            greater than or equal to the proposed spending
            limit.

            This field is a member of `oneof`_ ``approved_spending_limit``.
        approved_spending_limit_type (google.ads.googleads.v22.enums.types.SpendingLimitTypeEnum.SpendingLimitType):
            Output only. The approved spending limit as a
            well-defined type, for example, INFINITE.  This
            will only be populated if the approved spending
            limit is INFINITE.

            This field is a member of `oneof`_ ``approved_spending_limit``.
        adjusted_spending_limit_micros (int):
            Output only. The adjusted spending limit in
            micros.  One million is equivalent to one unit.

            If the approved spending limit is finite, the
            adjusted spending limit may vary depending on
            the types of adjustments applied to this budget,
            if applicable.

            The different kinds of adjustments are described
            here:

            https://support.google.com/google-ads/answer/1704323

            For example, a debit adjustment reduces how much
            the account is allowed to spend.

            This field is a member of `oneof`_ ``adjusted_spending_limit``.
        adjusted_spending_limit_type (google.ads.googleads.v22.enums.types.SpendingLimitTypeEnum.SpendingLimitType):
            Output only. The adjusted spending limit as a
            well-defined type, for example, INFINITE. This
            will only be populated if the adjusted spending
            limit is INFINITE, which is guaranteed to be
            true if the approved spending limit is INFINITE.

            This field is a member of `oneof`_ ``adjusted_spending_limit``.
    """

    class PendingAccountBudgetProposal(proto.Message):
        r"""A pending proposal associated with the enclosing
        account-level budget, if applicable.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            account_budget_proposal (str):
                Output only. The resource name of the proposal.
                AccountBudgetProposal resource names have the form:

                ``customers/{customer_id}/accountBudgetProposals/{account_budget_proposal_id}``

                This field is a member of `oneof`_ ``_account_budget_proposal``.
            proposal_type (google.ads.googleads.v22.enums.types.AccountBudgetProposalTypeEnum.AccountBudgetProposalType):
                Output only. The type of this proposal, for
                example, END to end the budget associated with
                this proposal.
            name (str):
                Output only. The name to assign to the
                account-level budget.

                This field is a member of `oneof`_ ``_name``.
            start_date_time (str):
                Output only. The start time in yyyy-MM-dd
                HH:mm:ss format.

                This field is a member of `oneof`_ ``_start_date_time``.
            purchase_order_number (str):
                Output only. A purchase order number is a
                value that helps users reference this budget in
                their monthly invoices.

                This field is a member of `oneof`_ ``_purchase_order_number``.
            notes (str):
                Output only. Notes associated with this
                budget.

                This field is a member of `oneof`_ ``_notes``.
            creation_date_time (str):
                Output only. The time when this account-level
                budget proposal was created. Formatted as
                yyyy-MM-dd HH:mm:ss.

                This field is a member of `oneof`_ ``_creation_date_time``.
            end_date_time (str):
                Output only. The end time in yyyy-MM-dd
                HH:mm:ss format.

                This field is a member of `oneof`_ ``end_time``.
            end_time_type (google.ads.googleads.v22.enums.types.TimeTypeEnum.TimeType):
                Output only. The end time as a well-defined
                type, for example, FOREVER.

                This field is a member of `oneof`_ ``end_time``.
            spending_limit_micros (int):
                Output only. The spending limit in micros.
                One million is equivalent to one unit.

                This field is a member of `oneof`_ ``spending_limit``.
            spending_limit_type (google.ads.googleads.v22.enums.types.SpendingLimitTypeEnum.SpendingLimitType):
                Output only. The spending limit as a
                well-defined type, for example, INFINITE.

                This field is a member of `oneof`_ ``spending_limit``.
        """

        account_budget_proposal: str = proto.Field(
            proto.STRING,
            number=12,
            optional=True,
        )
        proposal_type: (
            account_budget_proposal_type.AccountBudgetProposalTypeEnum.AccountBudgetProposalType
        ) = proto.Field(
            proto.ENUM,
            number=2,
            enum=account_budget_proposal_type.AccountBudgetProposalTypeEnum.AccountBudgetProposalType,
        )
        name: str = proto.Field(
            proto.STRING,
            number=13,
            optional=True,
        )
        start_date_time: str = proto.Field(
            proto.STRING,
            number=14,
            optional=True,
        )
        purchase_order_number: str = proto.Field(
            proto.STRING,
            number=17,
            optional=True,
        )
        notes: str = proto.Field(
            proto.STRING,
            number=18,
            optional=True,
        )
        creation_date_time: str = proto.Field(
            proto.STRING,
            number=19,
            optional=True,
        )
        end_date_time: str = proto.Field(
            proto.STRING,
            number=15,
            oneof="end_time",
        )
        end_time_type: time_type.TimeTypeEnum.TimeType = proto.Field(
            proto.ENUM,
            number=6,
            oneof="end_time",
            enum=time_type.TimeTypeEnum.TimeType,
        )
        spending_limit_micros: int = proto.Field(
            proto.INT64,
            number=16,
            oneof="spending_limit",
        )
        spending_limit_type: (
            gage_spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType
        ) = proto.Field(
            proto.ENUM,
            number=8,
            oneof="spending_limit",
            enum=gage_spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType,
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=23,
        optional=True,
    )
    billing_setup: str = proto.Field(
        proto.STRING,
        number=24,
        optional=True,
    )
    status: (
        account_budget_status.AccountBudgetStatusEnum.AccountBudgetStatus
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=account_budget_status.AccountBudgetStatusEnum.AccountBudgetStatus,
    )
    name: str = proto.Field(
        proto.STRING,
        number=25,
        optional=True,
    )
    proposed_start_date_time: str = proto.Field(
        proto.STRING,
        number=26,
        optional=True,
    )
    approved_start_date_time: str = proto.Field(
        proto.STRING,
        number=27,
        optional=True,
    )
    total_adjustments_micros: int = proto.Field(
        proto.INT64,
        number=33,
    )
    amount_served_micros: int = proto.Field(
        proto.INT64,
        number=34,
    )
    purchase_order_number: str = proto.Field(
        proto.STRING,
        number=35,
        optional=True,
    )
    notes: str = proto.Field(
        proto.STRING,
        number=36,
        optional=True,
    )
    pending_proposal: PendingAccountBudgetProposal = proto.Field(
        proto.MESSAGE,
        number=22,
        message=PendingAccountBudgetProposal,
    )
    proposed_end_date_time: str = proto.Field(
        proto.STRING,
        number=28,
        oneof="proposed_end_time",
    )
    proposed_end_time_type: time_type.TimeTypeEnum.TimeType = proto.Field(
        proto.ENUM,
        number=9,
        oneof="proposed_end_time",
        enum=time_type.TimeTypeEnum.TimeType,
    )
    approved_end_date_time: str = proto.Field(
        proto.STRING,
        number=29,
        oneof="approved_end_time",
    )
    approved_end_time_type: time_type.TimeTypeEnum.TimeType = proto.Field(
        proto.ENUM,
        number=11,
        oneof="approved_end_time",
        enum=time_type.TimeTypeEnum.TimeType,
    )
    proposed_spending_limit_micros: int = proto.Field(
        proto.INT64,
        number=30,
        oneof="proposed_spending_limit",
    )
    proposed_spending_limit_type: (
        gage_spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType
    ) = proto.Field(
        proto.ENUM,
        number=13,
        oneof="proposed_spending_limit",
        enum=gage_spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType,
    )
    approved_spending_limit_micros: int = proto.Field(
        proto.INT64,
        number=31,
        oneof="approved_spending_limit",
    )
    approved_spending_limit_type: (
        gage_spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType
    ) = proto.Field(
        proto.ENUM,
        number=15,
        oneof="approved_spending_limit",
        enum=gage_spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType,
    )
    adjusted_spending_limit_micros: int = proto.Field(
        proto.INT64,
        number=32,
        oneof="adjusted_spending_limit",
    )
    adjusted_spending_limit_type: (
        gage_spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType
    ) = proto.Field(
        proto.ENUM,
        number=17,
        oneof="adjusted_spending_limit",
        enum=gage_spending_limit_type.SpendingLimitTypeEnum.SpendingLimitType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "ReachPlanConversionRateModelEnum",
    },
)


class ReachPlanConversionRateModelEnum(proto.Message):
    r"""Container for enum describing the type of model used to
    create a conversion rate suggestion for a supported ad product.

    """

    class ReachPlanConversionRateModel(proto.Enum):
        r"""Types of models used to create conversion rate suggestions.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            CUSTOMER_HISTORY (2):
                Suggested conversion rate for the
                authenticated customer based on the previous 70
                days.
            INVENTORY_AGGRESSIVE (3):
                Suggested conversion rate based on an
                aggressive rate for the entire inventory.
            INVENTORY_CONSERVATIVE (4):
                Suggested conversion rate based on a
                conservative rate for the entire inventory.
            INVENTORY_MEDIAN (5):
                Suggested conversion rate based on the median
                rate for the entire inventory.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CUSTOMER_HISTORY = 2
        INVENTORY_AGGRESSIVE = 3
        INVENTORY_CONSERVATIVE = 4
        INVENTORY_MEDIAN = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "LanguageConstant",
    },
)


class LanguageConstant(proto.Message):
    r"""A language.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the language constant.
            Language constant resource names have the form:

            ``languageConstants/{criterion_id}``
        id (int):
            Output only. The ID of the language constant.

            This field is a member of `oneof`_ ``_id``.
        code (str):
            Output only. The language code, for example, "en_US",
            "en_AU", "es", "fr", etc.

            This field is a member of `oneof`_ ``_code``.
        name (str):
            Output only. The full name of the language in
            English, for example, "English (US)", "Spanish",
            etc.

            This field is a member of `oneof`_ ``_name``.
        targetable (bool):
            Output only. Whether the language is
            targetable.

            This field is a member of `oneof`_ ``_targetable``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=6,
        optional=True,
    )
    code: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    targetable: bool = proto.Field(
        proto.BOOL,
        number=9,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "MultiplierErrorEnum",
    },
)


class MultiplierErrorEnum(proto.Message):
    r"""Container for enum describing possible multiplier errors."""

    class MultiplierError(proto.Enum):
        r"""Enum describing possible multiplier errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            MULTIPLIER_TOO_HIGH (2):
                Multiplier value is too high
            MULTIPLIER_TOO_LOW (3):
                Multiplier value is too low
            TOO_MANY_FRACTIONAL_DIGITS (4):
                Too many fractional digits
            MULTIPLIER_NOT_ALLOWED_FOR_BIDDING_STRATEGY (5):
                A multiplier cannot be set for this bidding
                strategy
            MULTIPLIER_NOT_ALLOWED_WHEN_BASE_BID_IS_MISSING (6):
                A multiplier cannot be set when there is no
                base bid (for example, content max cpc)
            NO_MULTIPLIER_SPECIFIED (7):
                A bid multiplier must be specified
            MULTIPLIER_CAUSES_BID_TO_EXCEED_DAILY_BUDGET (8):
                Multiplier causes bid to exceed daily budget
            MULTIPLIER_CAUSES_BID_TO_EXCEED_MONTHLY_BUDGET (9):
                Multiplier causes bid to exceed monthly
                budget
            MULTIPLIER_CAUSES_BID_TO_EXCEED_CUSTOM_BUDGET (10):
                Multiplier causes bid to exceed custom budget
            MULTIPLIER_CAUSES_BID_TO_EXCEED_MAX_ALLOWED_BID (11):
                Multiplier causes bid to exceed maximum
                allowed bid
            BID_LESS_THAN_MIN_ALLOWED_BID_WITH_MULTIPLIER (12):
                Multiplier causes bid to become less than the
                minimum bid allowed
            MULTIPLIER_AND_BIDDING_STRATEGY_TYPE_MISMATCH (13):
                Multiplier type (cpc versus cpm) needs to
                match campaign's bidding strategy
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        MULTIPLIER_TOO_HIGH = 2
        MULTIPLIER_TOO_LOW = 3
        TOO_MANY_FRACTIONAL_DIGITS = 4
        MULTIPLIER_NOT_ALLOWED_FOR_BIDDING_STRATEGY = 5
        MULTIPLIER_NOT_ALLOWED_WHEN_BASE_BID_IS_MISSING = 6
        NO_MULTIPLIER_SPECIFIED = 7
        MULTIPLIER_CAUSES_BID_TO_EXCEED_DAILY_BUDGET = 8
        MULTIPLIER_CAUSES_BID_TO_EXCEED_MONTHLY_BUDGET = 9
        MULTIPLIER_CAUSES_BID_TO_EXCEED_CUSTOM_BUDGET = 10
        MULTIPLIER_CAUSES_BID_TO_EXCEED_MAX_ALLOWED_BID = 11
        BID_LESS_THAN_MIN_ALLOWED_BID_WITH_MULTIPLIER = 12
        MULTIPLIER_AND_BIDDING_STRATEGY_TYPE_MISMATCH = 13


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.common.types import criteria
from google.ads.googleads.v19.enums.types import (
    bid_modifier_source as gage_bid_modifier_source,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "AdGroupBidModifier",
    },
)


class AdGroupBidModifier(proto.Message):
    r"""Represents an ad group bid modifier.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the ad group bid modifier.
            Ad group bid modifier resource names have the form:

            ``customers/{customer_id}/adGroupBidModifiers/{ad_group_id}~{criterion_id}``
        ad_group (str):
            Immutable. The ad group to which this
            criterion belongs.

            This field is a member of `oneof`_ ``_ad_group``.
        criterion_id (int):
            Output only. The ID of the criterion to bid
            modify.
            This field is ignored for mutates.

            This field is a member of `oneof`_ ``_criterion_id``.
        bid_modifier (float):
            The modifier for the bid when the criterion
            matches. The modifier must be in the range: 0.1
            - 10.0. The range is 1.0 - 6.0 for
            PreferredContent. Use 0 to opt out of a Device
            type.

            This field is a member of `oneof`_ ``_bid_modifier``.
        base_ad_group (str):
            Output only. The base ad group from which this draft/trial
            adgroup bid modifier was created. If ad_group is a base ad
            group then this field will be equal to ad_group. If the ad
            group was created in the draft or trial and has no
            corresponding base ad group, then this field will be null.
            This field is readonly.

            This field is a member of `oneof`_ ``_base_ad_group``.
        bid_modifier_source (google.ads.googleads.v19.enums.types.BidModifierSourceEnum.BidModifierSource):
            Output only. Bid modifier source.
        hotel_date_selection_type (google.ads.googleads.v19.common.types.HotelDateSelectionTypeInfo):
            Immutable. Criterion for hotel date selection
            (default dates versus user selected).

            This field is a member of `oneof`_ ``criterion``.
        hotel_advance_booking_window (google.ads.googleads.v19.common.types.HotelAdvanceBookingWindowInfo):
            Immutable. Criterion for number of days prior
            to the stay the booking is being made.

            This field is a member of `oneof`_ ``criterion``.
        hotel_length_of_stay (google.ads.googleads.v19.common.types.HotelLengthOfStayInfo):
            Immutable. Criterion for length of hotel stay
            in nights.

            This field is a member of `oneof`_ ``criterion``.
        hotel_check_in_day (google.ads.googleads.v19.common.types.HotelCheckInDayInfo):
            Immutable. Criterion for day of the week the
            booking is for.

            This field is a member of `oneof`_ ``criterion``.
        device (google.ads.googleads.v19.common.types.DeviceInfo):
            Immutable. A device criterion.

            This field is a member of `oneof`_ ``criterion``.
        hotel_check_in_date_range (google.ads.googleads.v19.common.types.HotelCheckInDateRangeInfo):
            Immutable. Criterion for a hotel check-in
            date range.

            This field is a member of `oneof`_ ``criterion``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    ad_group: str = proto.Field(
        proto.STRING,
        number=13,
        optional=True,
    )
    criterion_id: int = proto.Field(
        proto.INT64,
        number=14,
        optional=True,
    )
    bid_modifier: float = proto.Field(
        proto.DOUBLE,
        number=15,
        optional=True,
    )
    base_ad_group: str = proto.Field(
        proto.STRING,
        number=16,
        optional=True,
    )
    bid_modifier_source: (
        gage_bid_modifier_source.BidModifierSourceEnum.BidModifierSource
    ) = proto.Field(
        proto.ENUM,
        number=10,
        enum=gage_bid_modifier_source.BidModifierSourceEnum.BidModifierSource,
    )
    hotel_date_selection_type: criteria.HotelDateSelectionTypeInfo = (
        proto.Field(
            proto.MESSAGE,
            number=5,
            oneof="criterion",
            message=criteria.HotelDateSelectionTypeInfo,
        )
    )
    hotel_advance_booking_window: criteria.HotelAdvanceBookingWindowInfo = (
        proto.Field(
            proto.MESSAGE,
            number=6,
            oneof="criterion",
            message=criteria.HotelAdvanceBookingWindowInfo,
        )
    )
    hotel_length_of_stay: criteria.HotelLengthOfStayInfo = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="criterion",
        message=criteria.HotelLengthOfStayInfo,
    )
    hotel_check_in_day: criteria.HotelCheckInDayInfo = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="criterion",
        message=criteria.HotelCheckInDayInfo,
    )
    device: criteria.DeviceInfo = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="criterion",
        message=criteria.DeviceInfo,
    )
    hotel_check_in_date_range: criteria.HotelCheckInDateRangeInfo = proto.Field(
        proto.MESSAGE,
        number=17,
        oneof="criterion",
        message=criteria.HotelCheckInDateRangeInfo,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "LocalServicesLicenseRejectionReasonEnum",
    },
)


class LocalServicesLicenseRejectionReasonEnum(proto.Message):
    r"""Container for enum describing the rejection reason of a local
    services license verification artifact.

    """

    class LocalServicesLicenseRejectionReason(proto.Enum):
        r"""Enums describing possible rejection reasons of a local
        services license verification artifact.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            BUSINESS_NAME_MISMATCH (2):
                Business name doesn't match business name for
                the Local Services Ad.
            UNAUTHORIZED (3):
                License is unauthorized or been revoked.
            EXPIRED (4):
                License is expired.
            POOR_QUALITY (5):
                License is poor quality - blurry images,
                illegible, etc...
            UNVERIFIABLE (6):
                License cannot be verified due to a not
                legitimate image.
            WRONG_DOCUMENT_OR_ID (7):
                License is not the requested document type or
                contains an invalid ID.
            OTHER (8):
                License has another flaw not listed
                explicitly.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        BUSINESS_NAME_MISMATCH = 2
        UNAUTHORIZED = 3
        EXPIRED = 4
        POOR_QUALITY = 5
        UNVERIFIABLE = 6
        WRONG_DOCUMENT_OR_ID = 7
        OTHER = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "GeoTargetConstantSuggestionErrorEnum",
    },
)


class GeoTargetConstantSuggestionErrorEnum(proto.Message):
    r"""Container for enum describing possible geo target constant
    suggestion errors.

    """

    class GeoTargetConstantSuggestionError(proto.Enum):
        r"""Enum describing possible geo target constant suggestion
        errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            LOCATION_NAME_SIZE_LIMIT (2):
                A location name cannot be greater than 300
                characters.
            LOCATION_NAME_LIMIT (3):
                At most 25 location names can be specified in
                a SuggestGeoTargetConstants method.
            INVALID_COUNTRY_CODE (4):
                The country code is invalid.
            REQUEST_PARAMETERS_UNSET (5):
                Geo target constant resource names or
                location names must be provided in the request.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        LOCATION_NAME_SIZE_LIMIT = 2
        LOCATION_NAME_LIMIT = 3
        INVALID_COUNTRY_CODE = 4
        REQUEST_PARAMETERS_UNSET = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "BrandStateEnum",
    },
)


class BrandStateEnum(proto.Message):
    r"""Container for enum describing possible brand states."""

    class BrandState(proto.Enum):
        r"""Enumeration of different brand states.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ENABLED (2):
                Brand is verified and globally available for
                selection
            DEPRECATED (3):
                Brand was globally available in past but is
                no longer a valid brand (based on business
                criteria)
            UNVERIFIED (4):
                Brand is unverified and customer scoped, but
                can be selected by customer (only who requested
                for same) for targeting
            APPROVED (5):
                Was a customer-scoped (unverified) brand,
                which got approved by business and added to the
                global list. Its assigned CKG MID should be used
                instead of this
            CANCELLED (6):
                Was a customer-scoped (unverified) brand, but
                the request was canceled by customer and this
                brand id is no longer valid
            REJECTED (7):
                Was a customer-scoped (unverified) brand, but
                the request was rejected by internal business
                team and this brand id is no longer valid
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ENABLED = 2
        DEPRECATED = 3
        UNVERIFIED = 4
        APPROVED = 5
        CANCELLED = 6
        REJECTED = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "BillingSetupErrorEnum",
    },
)


class BillingSetupErrorEnum(proto.Message):
    r"""Container for enum describing possible billing setup errors."""

    class BillingSetupError(proto.Enum):
        r"""Enum describing possible billing setup errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CANNOT_USE_EXISTING_AND_NEW_ACCOUNT (2):
                Cannot specify both an existing payments
                account and a new payments account when setting
                up billing.
            CANNOT_REMOVE_STARTED_BILLING_SETUP (3):
                Cannot cancel an approved billing setup whose
                start time has passed.
            CANNOT_CHANGE_BILLING_TO_SAME_PAYMENTS_ACCOUNT (4):
                Cannot perform a Change of Bill-To (CBT) to
                the same payments account.
            BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_STATUS (5):
                Billing setups can only be used by customers
                with ENABLED or DRAFT status.
            INVALID_PAYMENTS_ACCOUNT (6):
                Billing setups must either include a
                correctly formatted existing payments account
                id, or a non-empty new payments account name.
            BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_CATEGORY (7):
                Only billable and third-party customers can
                create billing setups.
            INVALID_START_TIME_TYPE (8):
                Billing setup creations can only use NOW for
                start time type.
            THIRD_PARTY_ALREADY_HAS_BILLING (9):
                Billing setups can only be created for a
                third-party customer if they do not already have
                a setup.
            BILLING_SETUP_IN_PROGRESS (10):
                Billing setups cannot be created if there is
                already a pending billing in progress.
            NO_SIGNUP_PERMISSION (11):
                Billing setups can only be created by
                customers who have permission to setup billings.
                Users can contact a representative for help
                setting up permissions.
            CHANGE_OF_BILL_TO_IN_PROGRESS (12):
                Billing setups cannot be created if there is
                already a future-approved billing.
            PAYMENTS_PROFILE_NOT_FOUND (13):
                Requested payments profile not found.
            PAYMENTS_ACCOUNT_NOT_FOUND (14):
                Requested payments account not found.
            PAYMENTS_PROFILE_INELIGIBLE (15):
                Billing setup creation failed because the
                payments profile is ineligible.
            PAYMENTS_ACCOUNT_INELIGIBLE (16):
                Billing setup creation failed because the
                payments account is ineligible.
            CUSTOMER_NEEDS_INTERNAL_APPROVAL (17):
                Billing setup creation failed because the
                payments profile needs internal approval.
            PAYMENTS_PROFILE_NEEDS_SERVICE_AGREEMENT_ACCEPTANCE (18):
                Billing setup creation failed because the
                user needs to accept master service agreement on
                the payments profile.
            PAYMENTS_ACCOUNT_INELIGIBLE_CURRENCY_CODE_MISMATCH (19):
                Payments account has different currency code
                than the current customer and hence cannot be
                used to setup billing.
            FUTURE_START_TIME_PROHIBITED (20):
                A start time in the future cannot be used
                because there is currently no active billing
                setup for this customer.
            TOO_MANY_BILLING_SETUPS_FOR_PAYMENTS_ACCOUNT (21):
                The payments account has maximum number of
                billing setups.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CANNOT_USE_EXISTING_AND_NEW_ACCOUNT = 2
        CANNOT_REMOVE_STARTED_BILLING_SETUP = 3
        CANNOT_CHANGE_BILLING_TO_SAME_PAYMENTS_ACCOUNT = 4
        BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_STATUS = 5
        INVALID_PAYMENTS_ACCOUNT = 6
        BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_CATEGORY = 7
        INVALID_START_TIME_TYPE = 8
        THIRD_PARTY_ALREADY_HAS_BILLING = 9
        BILLING_SETUP_IN_PROGRESS = 10
        NO_SIGNUP_PERMISSION = 11
        CHANGE_OF_BILL_TO_IN_PROGRESS = 12
        PAYMENTS_PROFILE_NOT_FOUND = 13
        PAYMENTS_ACCOUNT_NOT_FOUND = 14
        PAYMENTS_PROFILE_INELIGIBLE = 15
        PAYMENTS_ACCOUNT_INELIGIBLE = 16
        CUSTOMER_NEEDS_INTERNAL_APPROVAL = 17
        PAYMENTS_PROFILE_NEEDS_SERVICE_AGREEMENT_ACCEPTANCE = 18
        PAYMENTS_ACCOUNT_INELIGIBLE_CURRENCY_CODE_MISMATCH = 19
        FUTURE_START_TIME_PROHIBITED = 20
        TOO_MANY_BILLING_SETUPS_FOR_PAYMENTS_ACCOUNT = 21


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import (
    served_asset_field_type as gage_served_asset_field_type,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.common",
    marshal="google.ads.googleads.v21",
    manifest={
        "AssetUsage",
    },
)


class AssetUsage(proto.Message):
    r"""Contains the usage information of the asset.

    Attributes:
        asset (str):
            Resource name of the asset.
        served_asset_field_type (google.ads.googleads.v21.enums.types.ServedAssetFieldTypeEnum.ServedAssetFieldType):
            The served field type of the asset.
    """

    asset: str = proto.Field(
        proto.STRING,
        number=1,
    )
    served_asset_field_type: (
        gage_served_asset_field_type.ServedAssetFieldTypeEnum.ServedAssetFieldType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=gage_served_asset_field_type.ServedAssetFieldTypeEnum.ServedAssetFieldType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "ClickTypeEnum",
    },
)


class ClickTypeEnum(proto.Message):
    r"""Container for enumeration of Google Ads click types."""

    class ClickType(proto.Enum):
        r"""Enumerates Google Ads click types.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            APP_DEEPLINK (2):
                App engagement ad deep link.
            BREADCRUMBS (3):
                Breadcrumbs.
            BROADBAND_PLAN (4):
                Broadband Plan.
            CALL_TRACKING (5):
                Manually dialed phone calls.
            CALLS (6):
                Phone calls.
            CLICK_ON_ENGAGEMENT_AD (7):
                Click on engagement ad.
            GET_DIRECTIONS (8):
                Driving direction.
            LOCATION_EXPANSION (9):
                Get location details.
            LOCATION_FORMAT_CALL (10):
                Call.
            LOCATION_FORMAT_DIRECTIONS (11):
                Directions.
            LOCATION_FORMAT_IMAGE (12):
                Image(s).
            LOCATION_FORMAT_LANDING_PAGE (13):
                Go to landing page.
            LOCATION_FORMAT_MAP (14):
                Map.
            LOCATION_FORMAT_STORE_INFO (15):
                Go to store info.
            LOCATION_FORMAT_TEXT (16):
                Text.
            MOBILE_CALL_TRACKING (17):
                Mobile phone calls.
            OFFER_PRINTS (18):
                Print offer.
            OTHER (19):
                Other.
            PRODUCT_EXTENSION_CLICKS (20):
                Product plusbox offer.
            PRODUCT_LISTING_AD_CLICKS (21):
                Shopping - Product - Online.
            SITELINKS (22):
                Sitelink.
            STORE_LOCATOR (23):
                Show nearby locations.
            URL_CLICKS (25):
                Headline.
            VIDEO_APP_STORE_CLICKS (26):
                App store.
            VIDEO_CALL_TO_ACTION_CLICKS (27):
                Call-to-Action overlay.
            VIDEO_CARD_ACTION_HEADLINE_CLICKS (28):
                Cards.
            VIDEO_END_CAP_CLICKS (29):
                End cap.
            VIDEO_WEBSITE_CLICKS (30):
                Website.
            VISUAL_SITELINKS (31):
                Visual Sitelinks.
            WIRELESS_PLAN (32):
                Wireless Plan.
            PRODUCT_LISTING_AD_LOCAL (33):
                Shopping - Product - Local.
            PRODUCT_LISTING_AD_MULTICHANNEL_LOCAL (34):
                Shopping - Product - MultiChannel Local.
            PRODUCT_LISTING_AD_MULTICHANNEL_ONLINE (35):
                Shopping - Product - MultiChannel Online.
            PRODUCT_LISTING_ADS_COUPON (36):
                Shopping - Product - Coupon.
            PRODUCT_LISTING_AD_TRANSACTABLE (37):
                Shopping - Product - Sell on Google.
            PRODUCT_AD_APP_DEEPLINK (38):
                Shopping - Product - App engagement ad deep
                link.
            SHOWCASE_AD_CATEGORY_LINK (39):
                Shopping - Showcase - Category.
            SHOWCASE_AD_LOCAL_STOREFRONT_LINK (40):
                Shopping - Showcase - Local storefront.
            SHOWCASE_AD_ONLINE_PRODUCT_LINK (42):
                Shopping - Showcase - Online product.
            SHOWCASE_AD_LOCAL_PRODUCT_LINK (43):
                Shopping - Showcase - Local product.
            PROMOTION_EXTENSION (44):
                Promotion Extension.
            SWIPEABLE_GALLERY_AD_HEADLINE (45):
                Ad Headline.
            SWIPEABLE_GALLERY_AD_SWIPES (46):
                Swipes.
            SWIPEABLE_GALLERY_AD_SEE_MORE (47):
                See More.
            SWIPEABLE_GALLERY_AD_SITELINK_ONE (48):
                Sitelink 1.
            SWIPEABLE_GALLERY_AD_SITELINK_TWO (49):
                Sitelink 2.
            SWIPEABLE_GALLERY_AD_SITELINK_THREE (50):
                Sitelink 3.
            SWIPEABLE_GALLERY_AD_SITELINK_FOUR (51):
                Sitelink 4.
            SWIPEABLE_GALLERY_AD_SITELINK_FIVE (52):
                Sitelink 5.
            HOTEL_PRICE (53):
                Hotel price.
            PRICE_EXTENSION (54):
                Price Extension.
            HOTEL_BOOK_ON_GOOGLE_ROOM_SELECTION (55):
                Book on Google hotel room selection.
            SHOPPING_COMPARISON_LISTING (56):
                Shopping - Comparison Listing.
            CROSS_NETWORK (57):
                Cross-network. From Performance Max and
                Discovery Campaigns.
            AD_IMAGE (58):
                AD Images.
            TRAVEL_ASSETS (59):
                Travel Feed Assets.
            VEHICLE_ASSETS (60):
                Vehicle Feed Assets.
            PRODUCT_ASSETS (61):
                Product Feed Assets.
            VIDEO_CHANNEL_CLICK (62):
                Click on channel icon that navigates the user
                to the corresponding YouTube channel.
            VIDEO_RELATED_VIDEOS_CLICK (63):
                Click on a related video that navigates the
                user to the video watch page.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        APP_DEEPLINK = 2
        BREADCRUMBS = 3
        BROADBAND_PLAN = 4
        CALL_TRACKING = 5
        CALLS = 6
        CLICK_ON_ENGAGEMENT_AD = 7
        GET_DIRECTIONS = 8
        LOCATION_EXPANSION = 9
        LOCATION_FORMAT_CALL = 10
        LOCATION_FORMAT_DIRECTIONS = 11
        LOCATION_FORMAT_IMAGE = 12
        LOCATION_FORMAT_LANDING_PAGE = 13
        LOCATION_FORMAT_MAP = 14
        LOCATION_FORMAT_STORE_INFO = 15
        LOCATION_FORMAT_TEXT = 16
        MOBILE_CALL_TRACKING = 17
        OFFER_PRINTS = 18
        OTHER = 19
        PRODUCT_EXTENSION_CLICKS = 20
        PRODUCT_LISTING_AD_CLICKS = 21
        SITELINKS = 22
        STORE_LOCATOR = 23
        URL_CLICKS = 25
        VIDEO_APP_STORE_CLICKS = 26
        VIDEO_CALL_TO_ACTION_CLICKS = 27
        VIDEO_CARD_ACTION_HEADLINE_CLICKS = 28
        VIDEO_END_CAP_CLICKS = 29
        VIDEO_WEBSITE_CLICKS = 30
        VISUAL_SITELINKS = 31
        WIRELESS_PLAN = 32
        PRODUCT_LISTING_AD_LOCAL = 33
        PRODUCT_LISTING_AD_MULTICHANNEL_LOCAL = 34
        PRODUCT_LISTING_AD_MULTICHANNEL_ONLINE = 35
        PRODUCT_LISTING_ADS_COUPON = 36
        PRODUCT_LISTING_AD_TRANSACTABLE = 37
        PRODUCT_AD_APP_DEEPLINK = 38
        SHOWCASE_AD_CATEGORY_LINK = 39
        SHOWCASE_AD_LOCAL_STOREFRONT_LINK = 40
        SHOWCASE_AD_ONLINE_PRODUCT_LINK = 42
        SHOWCASE_AD_LOCAL_PRODUCT_LINK = 43
        PROMOTION_EXTENSION = 44
        SWIPEABLE_GALLERY_AD_HEADLINE = 45
        SWIPEABLE_GALLERY_AD_SWIPES = 46
        SWIPEABLE_GALLERY_AD_SEE_MORE = 47
        SWIPEABLE_GALLERY_AD_SITELINK_ONE = 48
        SWIPEABLE_GALLERY_AD_SITELINK_TWO = 49
        SWIPEABLE_GALLERY_AD_SITELINK_THREE = 50
        SWIPEABLE_GALLERY_AD_SITELINK_FOUR = 51
        SWIPEABLE_GALLERY_AD_SITELINK_FIVE = 52
        HOTEL_PRICE = 53
        PRICE_EXTENSION = 54
        HOTEL_BOOK_ON_GOOGLE_ROOM_SELECTION = 55
        SHOPPING_COMPARISON_LISTING = 56
        CROSS_NETWORK = 57
        AD_IMAGE = 58
        TRAVEL_ASSETS = 59
        VEHICLE_ASSETS = 60
        PRODUCT_ASSETS = 61
        VIDEO_CHANNEL_CLICK = 62
        VIDEO_RELATED_VIDEOS_CLICK = 63


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import criteria
from google.ads.googleads.v22.enums.types import (
    asset_group_signal_approval_status,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "AssetGroupSignal",
    },
)


class AssetGroupSignal(proto.Message):
    r"""AssetGroupSignal represents a signal in an asset group. The
    existence of a signal tells the performance max campaign who's
    most likely to convert. Performance Max uses the signal to look
    for new people with similar or stronger intent to find
    conversions across Search, Display, Video, and more.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the asset group signal.
            Asset group signal resource name have the form:

            ``customers/{customer_id}/assetGroupSignals/{asset_group_id}~{signal_id}``
        asset_group (str):
            Immutable. The asset group which this asset
            group signal belongs to.
        approval_status (google.ads.googleads.v22.enums.types.AssetGroupSignalApprovalStatusEnum.AssetGroupSignalApprovalStatus):
            Output only. Approval status is the output
            value for search theme signal after Google ads
            policy review. When using Audience signal, this
            field is not used and will be absent.
        disapproval_reasons (MutableSequence[str]):
            Output only. Computed for SearchTheme
            signals. When using Audience signal, this field
            is not used and will be absent.
        audience (google.ads.googleads.v22.common.types.AudienceInfo):
            Immutable. The audience signal to be used by
            the performance max campaign.

            This field is a member of `oneof`_ ``signal``.
        search_theme (google.ads.googleads.v22.common.types.SearchThemeInfo):
            Immutable. The search_theme signal to be used by the
            performance max campaign. Mutate errors of search_theme
            criterion includes AssetGroupSignalError.UNSPECIFIED
            AssetGroupSignalError.UNKNOWN
            AssetGroupSignalError.TOO_MANY_WORDS
            AssetGroupSignalError.SEARCH_THEME_POLICY_VIOLATION
            FieldError.REQUIRED StringFormatError.ILLEGAL_CHARS
            StringLengthError.TOO_LONG
            ResourceCountLimitExceededError.RESOURCE_LIMIT

            This field is a member of `oneof`_ ``signal``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_group: str = proto.Field(
        proto.STRING,
        number=2,
    )
    approval_status: (
        asset_group_signal_approval_status.AssetGroupSignalApprovalStatusEnum.AssetGroupSignalApprovalStatus
    ) = proto.Field(
        proto.ENUM,
        number=6,
        enum=asset_group_signal_approval_status.AssetGroupSignalApprovalStatusEnum.AssetGroupSignalApprovalStatus,
    )
    disapproval_reasons: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=7,
    )
    audience: criteria.AudienceInfo = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="signal",
        message=criteria.AudienceInfo,
    )
    search_theme: criteria.SearchThemeInfo = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="signal",
        message=criteria.SearchThemeInfo,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "AttributionModelEnum",
    },
)


class AttributionModelEnum(proto.Message):
    r"""Container for enum representing the attribution model that
    describes how to distribute credit for a particular conversion
    across potentially many prior interactions.

    """

    class AttributionModel(proto.Enum):
        r"""The attribution model that describes how to distribute credit
        for a particular conversion across potentially many prior
        interactions.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            EXTERNAL (100):
                Uses external attribution.
            GOOGLE_ADS_LAST_CLICK (101):
                Attributes all credit for a conversion to its
                last click.
            GOOGLE_SEARCH_ATTRIBUTION_FIRST_CLICK (102):
                Attributes all credit for a conversion to its
                first click using Google Search attribution.
            GOOGLE_SEARCH_ATTRIBUTION_LINEAR (103):
                Attributes credit for a conversion equally
                across all of its clicks using Google Search
                attribution.
            GOOGLE_SEARCH_ATTRIBUTION_TIME_DECAY (104):
                Attributes exponentially more credit for a
                conversion to its more recent clicks using
                Google Search attribution (half-life is 1 week).
            GOOGLE_SEARCH_ATTRIBUTION_POSITION_BASED (105):
                Attributes 40% of the credit for a conversion
                to its first and last clicks. Remaining 20% is
                evenly distributed across all other clicks. This
                uses Google Search attribution.
            GOOGLE_SEARCH_ATTRIBUTION_DATA_DRIVEN (106):
                Flexible model that uses machine learning to
                determine the appropriate distribution of credit
                among clicks using Google Search attribution.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        EXTERNAL = 100
        GOOGLE_ADS_LAST_CLICK = 101
        GOOGLE_SEARCH_ATTRIBUTION_FIRST_CLICK = 102
        GOOGLE_SEARCH_ATTRIBUTION_LINEAR = 103
        GOOGLE_SEARCH_ATTRIBUTION_TIME_DECAY = 104
        GOOGLE_SEARCH_ATTRIBUTION_POSITION_BASED = 105
        GOOGLE_SEARCH_ATTRIBUTION_DATA_DRIVEN = 106


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "CustomerManagerLinkErrorEnum",
    },
)


class CustomerManagerLinkErrorEnum(proto.Message):
    r"""Container for enum describing possible CustomerManagerLink
    errors.

    """

    class CustomerManagerLinkError(proto.Enum):
        r"""Enum describing possible CustomerManagerLink errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            NO_PENDING_INVITE (2):
                No pending invitation.
            SAME_CLIENT_MORE_THAN_ONCE_PER_CALL (3):
                Attempt to operate on the same client more
                than once in the same call.
            MANAGER_HAS_MAX_NUMBER_OF_LINKED_ACCOUNTS (4):
                Manager account has the maximum number of
                linked accounts.
            CANNOT_UNLINK_ACCOUNT_WITHOUT_ACTIVE_USER (5):
                If no active user on account it cannot be
                unlinked from its manager.
            CANNOT_REMOVE_LAST_CLIENT_ACCOUNT_OWNER (6):
                Account should have at least one active owner
                on it before being unlinked.
            CANNOT_CHANGE_ROLE_BY_NON_ACCOUNT_OWNER (7):
                Only account owners may change their
                permission role.
            CANNOT_CHANGE_ROLE_FOR_NON_ACTIVE_LINK_ACCOUNT (8):
                When a client's link to its manager is not
                active, the link role cannot be changed.
            DUPLICATE_CHILD_FOUND (9):
                Attempt to link a child to a parent that
                contains or will contain duplicate children.
            TEST_ACCOUNT_LINKS_TOO_MANY_CHILD_ACCOUNTS (10):
                The authorized customer is a test account. It
                can add no more than the allowed number of
                accounts
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NO_PENDING_INVITE = 2
        SAME_CLIENT_MORE_THAN_ONCE_PER_CALL = 3
        MANAGER_HAS_MAX_NUMBER_OF_LINKED_ACCOUNTS = 4
        CANNOT_UNLINK_ACCOUNT_WITHOUT_ACTIVE_USER = 5
        CANNOT_REMOVE_LAST_CLIENT_ACCOUNT_OWNER = 6
        CANNOT_CHANGE_ROLE_BY_NON_ACCOUNT_OWNER = 7
        CANNOT_CHANGE_ROLE_FOR_NON_ACTIVE_LINK_ACCOUNT = 8
        DUPLICATE_CHILD_FOUND = 9
        TEST_ACCOUNT_LINKS_TOO_MANY_CHILD_ACCOUNTS = 10


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "LocalServicesLeadSurveyDissatisfiedReasonEnum",
    },
)


class LocalServicesLeadSurveyDissatisfiedReasonEnum(proto.Message):
    r"""Container for enum describing possible survey dissatisfied
    reasons for a lead.

    """

    class SurveyDissatisfiedReason(proto.Enum):
        r"""Provider's reason for not being satisfied with the lead.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            OTHER_DISSATISFIED_REASON (2):
                Other reasons.
            GEO_MISMATCH (3):
                Lead was for a service that does not match
                the business' service area.
            JOB_TYPE_MISMATCH (4):
                Lead was for a service that is not offered by
                the business.
            NOT_READY_TO_BOOK (5):
                Lead was by a customer that was not ready to
                book.
            SPAM (6):
                Lead was a spam. Example: lead was from a
                bot, silent called, scam, etc.
            DUPLICATE (7):
                Lead was a duplicate of another lead that is,
                customer contacted the business more than once.
            SOLICITATION (8):
                Lead due to solicitation. Example: a person
                trying to get a job or selling a product, etc.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        OTHER_DISSATISFIED_REASON = 2
        GEO_MISMATCH = 3
        JOB_TYPE_MISMATCH = 4
        NOT_READY_TO_BOOK = 5
        SPAM = 6
        DUPLICATE = 7
        SOLICITATION = 8


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example lists the resource names for the customers that the
authenticating user has access to.

The customer IDs retrieved from the resource names can be used to set
the login-customer-id configuration. For more information see this
documentation: https://developers.google.com/google-ads/api/docs/concepts/call-structure#cid
"""


import sys
from typing import List

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.customer_service.client import (
    CustomerServiceClient,
)
from google.ads.googleads.v22.services.types.customer_service import (
    ListAccessibleCustomersResponse,
)


# [START list_accessible_customers]
def main(client: GoogleAdsClient) -> None:
    customer_service: CustomerServiceClient = client.get_service(
        "CustomerService"
    )

    accessible_customers: ListAccessibleCustomersResponse = (
        customer_service.list_accessible_customers()
    )
    result_total: int = len(accessible_customers.resource_names)
    print(f"Total results: {result_total}")

    resource_names: List[str] = accessible_customers.resource_names
    for resource_name in resource_names:  # resource_name is implicitly str
        print(f'Customer resource name: "{resource_name}"')
    # [END list_accessible_customers]


if __name__ == "__main__":
    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(googleads_client)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "OptimizationGoalTypeEnum",
    },
)


class OptimizationGoalTypeEnum(proto.Message):
    r"""Container for enum describing the type of optimization goal."""

    class OptimizationGoalType(proto.Enum):
        r"""The type of optimization goal

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used as a return value only. Represents value
                unknown in this version.
            CALL_CLICKS (2):
                Optimize for call clicks. Call click
                conversions are times people selected 'Call' to
                contact a store after viewing an ad.
            DRIVING_DIRECTIONS (3):
                Optimize for driving directions. Driving
                directions conversions are times people selected
                'Get directions' to navigate to a store after
                viewing an ad.
            APP_PRE_REGISTRATION (4):
                Optimize for pre-registration.
                Pre-registration conversions are the number of
                pre-registration signups to receive a
                notification when the app is released.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CALL_CLICKS = 2
        DRIVING_DIRECTIONS = 3
        APP_PRE_REGISTRATION = 4


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This code example generates a video ads reach forecast."""

import argparse
import math
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.enums.types.reach_plan_age_range import (
    ReachPlanAgeRangeEnum,
)
from google.ads.googleads.v22.enums.types.gender_type import GenderTypeEnum
from google.ads.googleads.v22.enums.types.device import DeviceEnum
from google.ads.googleads.v22.common.types.criteria import (
    GenderInfo,
    DeviceInfo,
)
from google.ads.googleads.v22.services.services.reach_plan_service.client import (
    ReachPlanServiceClient,
)
from google.ads.googleads.v22.services.types.reach_plan_service import (
    ListPlannableLocationsResponse,
    PlannableLocation,
    ListPlannableProductsResponse,
    ProductMetadata,
    GenerateReachForecastRequest,
    GenerateReachForecastResponse,
    ReachForecast,
    PlannedProductReachForecast,
    PlannedProduct,
)

ONE_MILLION = 1.0e6


def main(client: GoogleAdsClient, customer_id: str):
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
    """
    # You can review a list of valid location IDs by visiting:
    # https://developers.google.com/google-ads/api/reference/data/geotargets
    # or by calling the ListPlannableLocations method on ReachPlanService.
    location_id = "2840"  # US
    currency_code = "USD"
    budget = 500000

    show_plannable_locations(client)
    show_plannable_products(client, location_id)
    forecast_manual_mix(client, customer_id, location_id, currency_code, budget)


def show_plannable_locations(client: GoogleAdsClient):
    """Shows map of plannable locations to their IDs.

    Args:
        client: an initialized GoogleAdsClient instance.
    """
    reach_plan_service: ReachPlanServiceClient = client.get_service(
        "ReachPlanService"
    )
    response: ListPlannableLocationsResponse = (
        reach_plan_service.list_plannable_locations()
    )

    print("Plannable Locations")
    print("Name,\tId,\tParentCountryId")
    location: PlannableLocation
    for location in response.plannable_locations:
        print(
            f"'{location.name}',\t{location.id},\t{location.parent_country_id}"
        )


# [START forecast_reach_2]
def show_plannable_products(client: GoogleAdsClient, location_id: str):
    """Lists plannable products for a given location.

    Args:
        client: an initialized GoogleAdsClient instance.
        location_id: The location ID to plan for.
    """
    reach_plan_service: ReachPlanServiceClient = client.get_service(
        "ReachPlanService"
    )
    response: ListPlannableProductsResponse = (
        reach_plan_service.list_plannable_products(
            plannable_location_id=location_id
        )
    )
    print(f"Plannable Products for Location ID {location_id}")

    product_metadata: ProductMetadata
    for product_metadata in response.product_metadata:
        print(
            f"{product_metadata.plannable_product_code} : "
            f"{product_metadata.plannable_product_name}"
        )

        print("Age Ranges:")
        age_range: ReachPlanAgeRangeEnum
        for age_range in product_metadata.plannable_targeting.age_ranges:
            print(f"\t- {age_range.name}")

        print("Genders:")
        gender: GenderInfo
        for gender in product_metadata.plannable_targeting.genders:
            print(f"\t- {gender.type_.name}")

        print("Devices:")
        device: DeviceInfo
        for device in product_metadata.plannable_targeting.devices:
            print(f"\t- {device.type_.name}")
        # [END forecast_reach_2]


# [START forecast_reach]
def request_reach_curve(
    client: GoogleAdsClient,
    customer_id: str,
    product_mix: list[PlannedProduct],
    location_id: str,
    currency_code: str,
):
    """Creates a sample request for a given product mix.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: The customer ID for the reach forecast.
        product_mix: The product mix for the reach forecast.
        location_id: The location ID to plan for.
        currency_code: Three-character ISO 4217 currency code.
    """
    # See the docs for defaults and valid ranges:
    # https://developers.google.com/google-ads/api/reference/rpc/latest/GenerateReachForecastRequest
    request: GenerateReachForecastRequest = client.get_type(
        "GenerateReachForecastRequest"
    )
    request.customer_id = customer_id
    # Valid durations are between 1 and 90 days.
    request.campaign_duration.duration_in_days = 28
    request.currency_code = currency_code
    request.cookie_frequency_cap = 0
    request.min_effective_frequency = 1
    request.planned_products = product_mix

    request.targeting.plannable_location_id = location_id
    request.targeting.age_range = (
        client.enums.ReachPlanAgeRangeEnum.AGE_RANGE_18_65_UP
    )

    # Add gender targeting to the request.
    gender_type: GenderTypeEnum
    for gender_type in [
        client.enums.GenderTypeEnum.FEMALE,
        client.enums.GenderTypeEnum.MALE,
    ]:
        gender: GenderInfo = client.get_type("GenderInfo")
        gender.type_ = gender_type
        request.targeting.genders.append(gender)

    # Add device targeting to the request.
    device_type: DeviceEnum
    for device_type in [
        client.enums.DeviceEnum.DESKTOP,
        client.enums.DeviceEnum.MOBILE,
        client.enums.DeviceEnum.TABLET,
    ]:
        device: DeviceInfo = client.get_type("DeviceInfo")
        device.type_ = device_type
        request.targeting.devices.append(device)

    reach_plan_service: ReachPlanServiceClient = client.get_service(
        "ReachPlanService"
    )
    response: GenerateReachForecastResponse = (
        reach_plan_service.generate_reach_forecast(request=request)
    )

    print(
        "Currency, Cost, On-Target Reach, On-Target Imprs, Total Reach,"
        " Total Imprs, Products"
    )
    point: ReachForecast
    for point in response.reach_curve.reach_forecasts:
        product_splits = []
        p: PlannedProductReachForecast
        for p in point.planned_product_reach_forecasts:
            product_splits.append(
                {p.plannable_product_code: p.cost_micros / ONE_MILLION}
            )
        print(
            [
                currency_code,
                point.cost_micros / ONE_MILLION,
                point.forecast.on_target_reach,
                point.forecast.on_target_impressions,
                point.forecast.total_reach,
                point.forecast.total_impressions,
                product_splits,
            ]
        )
        # [END forecast_reach]


# [START forecast_reach_3]
def forecast_manual_mix(
    client: GoogleAdsClient,
    customer_id: str,
    location_id: str,
    currency_code: str,
    budget: int,
):
    """Pulls a forecast for product mix created manually.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: The customer ID for the reach forecast.
        location_id: The location ID to plan for.
        currency_code: Three-character ISO 4217 currency code.
        budget: Budget to allocate to the plan.
    """
    product_mix: list[PlannedProduct] = []
    trueview_allocation = 0.15
    bumper_allocation = 1 - trueview_allocation
    product_splits = [
        ("TRUEVIEW_IN_STREAM", trueview_allocation),
        ("BUMPER", bumper_allocation),
    ]
    product: str
    split: float
    for product, split in product_splits:
        planned_product: PlannedProduct = client.get_type("PlannedProduct")
        planned_product.plannable_product_code = product
        planned_product.budget_micros = math.trunc(budget * ONE_MILLION * split)
        product_mix.append(planned_product)

    request_reach_curve(
        client, customer_id, product_mix, location_id, currency_code
    )
    # [END forecast_reach_3]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Generates video ads reach forecast."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            'Request with ID "{}" failed with status "%s" and includes the '
            "following errors:".format(ex.request_id, ex.error.code().name)
        )
        for error in ex.failure.errors:
            print('\tError with message "{}".'.format(error.message))
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(
                        "\t\tOn field: {}".format(field_path_element.field_name)
                    )
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.enums.types import asset_set_link_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "CustomerAssetSet",
    },
)


class CustomerAssetSet(proto.Message):
    r"""CustomerAssetSet is the linkage between a customer and an
    asset set. Adding a CustomerAssetSet links an asset set with a
    customer.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the customer asset set.
            Asset set asset resource names have the form:

            ``customers/{customer_id}/customerAssetSets/{asset_set_id}``
        asset_set (str):
            Immutable. The asset set which is linked to
            the customer.
        customer (str):
            Immutable. The customer to which this asset
            set is linked.
        status (google.ads.googleads.v19.enums.types.AssetSetLinkStatusEnum.AssetSetLinkStatus):
            Output only. The status of the customer asset
            set asset. Read-only.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_set: str = proto.Field(
        proto.STRING,
        number=2,
    )
    customer: str = proto.Field(
        proto.STRING,
        number=3,
    )
    status: asset_set_link_status.AssetSetLinkStatusEnum.AssetSetLinkStatus = (
        proto.Field(
            proto.ENUM,
            number=4,
            enum=asset_set_link_status.AssetSetLinkStatusEnum.AssetSetLinkStatus,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.common.types import additional_application_info
from google.ads.googleads.v21.common.types import audience_insights_attribute
from google.ads.googleads.v21.common.types import criteria
from google.ads.googleads.v21.enums.types import insights_trend


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "GenerateCreatorInsightsRequest",
        "GenerateCreatorInsightsResponse",
        "GenerateTrendingInsightsRequest",
        "GenerateTrendingInsightsResponse",
        "YouTubeCreatorInsights",
        "YouTubeMetrics",
        "YouTubeChannelInsights",
        "SearchAudience",
        "SearchTopics",
        "TrendInsight",
        "TrendInsightMetrics",
    },
)


class GenerateCreatorInsightsRequest(proto.Message):
    r"""Request message for
    [ContentCreatorInsightsService.GenerateCreatorInsights][google.ads.googleads.v21.services.ContentCreatorInsightsService.GenerateCreatorInsights].

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        customer_insights_group (str):
            Required. The name of the customer being
            planned for.  This is a user-defined value.
        insights_application_info (google.ads.googleads.v21.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
        country_locations (MutableSequence[google.ads.googleads.v21.common.types.LocationInfo]):
            Required. The countries to search that apply
            to the criteria.
        sub_country_locations (MutableSequence[google.ads.googleads.v21.common.types.LocationInfo]):
            The sub-country geographic locations to search that apply to
            the criteria. Only supported for [SearchAttributes]
            criteria.
        search_attributes (google.ads.googleads.v21.services.types.GenerateCreatorInsightsRequest.SearchAttributes):
            The attributes used to identify top creators. Data fetched
            is based on the list of countries or sub-country locations
            specified in [country_locations] or [sub_country_locations].

            This field is a member of `oneof`_ ``criteria``.
        search_brand (google.ads.googleads.v21.services.types.GenerateCreatorInsightsRequest.SearchBrand):
            A brand used to search for top creators. Data fetched is
            based on the list of countries specified in
            [country_locations].

            This field is a member of `oneof`_ ``criteria``.
        search_channels (google.ads.googleads.v21.services.types.GenerateCreatorInsightsRequest.YouTubeChannels):
            YouTube Channel IDs for Creator Insights. Data fetched for
            channels is based on the list of countries specified in
            [country_locations].

            This field is a member of `oneof`_ ``criteria``.
    """

    class SearchAttributes(proto.Message):
        r"""The audience attributes (such as Age, Gender, Affinity, and
        In-Market) and creator attributes (such as creator's content
        topics) used to search for top creators.

        Attributes:
            audience_attributes (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsAttribute]):
                Optional. Audience attributes that describe an audience of
                viewers. This is used to search for creators whose own
                viewers match the input audience. Attributes age_range,
                gender, user_interest, entity, category, device,
                parental_status, and income_range are supported. Attribute
                location is not supported.
            creator_attributes (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsAttribute]):
                Optional. Creator attributes that describe a collection of
                types of content. This is used to search for creators whose
                content matches the input creator attributes. Only Knowledge
                Graph Entities tagged with
                [InsightsKnowledgeGraphEntityCapabilities.CREATOR_ATTRIBUTE][]
                are supported. Use
                [AudienceInsightsService.ListAudienceInsightsAttributes][google.ads.googleads.v21.services.AudienceInsightsService.ListAudienceInsightsAttributes]
                to get the list of supported entities. Other attributes
                including location are not supported.
        """

        audience_attributes: MutableSequence[
            audience_insights_attribute.AudienceInsightsAttribute
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=audience_insights_attribute.AudienceInsightsAttribute,
        )
        creator_attributes: MutableSequence[
            audience_insights_attribute.AudienceInsightsAttribute
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message=audience_insights_attribute.AudienceInsightsAttribute,
        )

    class SearchBrand(proto.Message):
        r"""The brand used to search for top creators.

        Attributes:
            brand_entities (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsAttribute]):
                Optional. One or more Knowledge Graph
                Entities that represent the brand for which to
                find insights.
            include_related_topics (bool):
                Optional. When true, we will expand the search to beyond
                just the entities specified in [brand_entities] to other
                related knowledge graph entities similar to the brand. The
                default value is ``false``.
        """

        brand_entities: MutableSequence[
            audience_insights_attribute.AudienceInsightsAttribute
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=audience_insights_attribute.AudienceInsightsAttribute,
        )
        include_related_topics: bool = proto.Field(
            proto.BOOL,
            number=2,
        )

    class YouTubeChannels(proto.Message):
        r"""A collection of YouTube Channels.

        Attributes:
            youtube_channels (MutableSequence[google.ads.googleads.v21.common.types.YouTubeChannelInfo]):
                Optional. The YouTube Channel IDs to fetch
                creator insights for.
        """

        youtube_channels: MutableSequence[criteria.YouTubeChannelInfo] = (
            proto.RepeatedField(
                proto.MESSAGE,
                number=1,
                message=criteria.YouTubeChannelInfo,
            )
        )

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    customer_insights_group: str = proto.Field(
        proto.STRING,
        number=2,
    )
    insights_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=8,
        message=additional_application_info.AdditionalApplicationInfo,
    )
    country_locations: MutableSequence[criteria.LocationInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=6,
            message=criteria.LocationInfo,
        )
    )
    sub_country_locations: MutableSequence[criteria.LocationInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=7,
            message=criteria.LocationInfo,
        )
    )
    search_attributes: SearchAttributes = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="criteria",
        message=SearchAttributes,
    )
    search_brand: SearchBrand = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="criteria",
        message=SearchBrand,
    )
    search_channels: YouTubeChannels = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="criteria",
        message=YouTubeChannels,
    )


class GenerateCreatorInsightsResponse(proto.Message):
    r"""Response message for
    [ContentCreatorInsightsService.GenerateCreatorInsights][google.ads.googleads.v21.services.ContentCreatorInsightsService.GenerateCreatorInsights].

    Attributes:
        creator_insights (MutableSequence[google.ads.googleads.v21.services.types.YouTubeCreatorInsights]):
            A collection of YouTube Creators, each
            containing a collection of YouTube Channels
            maintained by the YouTube Creator.
    """

    creator_insights: MutableSequence["YouTubeCreatorInsights"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="YouTubeCreatorInsights",
        )
    )


class GenerateTrendingInsightsRequest(proto.Message):
    r"""Request message for
    [ContentCreatorInsightsService.GenerateTrendingInsights]

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        customer_insights_group (str):
            Required. The name of the customer being
            planned for. This is a user-defined value.
        insights_application_info (google.ads.googleads.v21.common.types.AdditionalApplicationInfo):
            Optional. Additional information on the
            application issuing the request.
        country_location (google.ads.googleads.v21.common.types.LocationInfo):
            Required. The country to find trends in.
        search_audience (google.ads.googleads.v21.services.types.SearchAudience):
            An audience to search for trending content
            in.

            This field is a member of `oneof`_ ``criteria``.
        search_topics (google.ads.googleads.v21.services.types.SearchTopics):
            Content topics to return trend information
            for.

            This field is a member of `oneof`_ ``criteria``.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    customer_insights_group: str = proto.Field(
        proto.STRING,
        number=2,
    )
    insights_application_info: (
        additional_application_info.AdditionalApplicationInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=6,
        message=additional_application_info.AdditionalApplicationInfo,
    )
    country_location: criteria.LocationInfo = proto.Field(
        proto.MESSAGE,
        number=3,
        message=criteria.LocationInfo,
    )
    search_audience: "SearchAudience" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="criteria",
        message="SearchAudience",
    )
    search_topics: "SearchTopics" = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="criteria",
        message="SearchTopics",
    )


class GenerateTrendingInsightsResponse(proto.Message):
    r"""Response message for
    [ContentCreatorInsightsService.GenerateTrendingInsights]

    Attributes:
        trend_insights (MutableSequence[google.ads.googleads.v21.services.types.TrendInsight]):
            The list of trending insights for the given
            criteria.
    """

    trend_insights: MutableSequence["TrendInsight"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="TrendInsight",
    )


class YouTubeCreatorInsights(proto.Message):
    r"""A YouTube creator and the insights for this creator.

    Attributes:
        creator_name (str):
            The name of the creator.
        creator_channels (MutableSequence[google.ads.googleads.v21.services.types.YouTubeChannelInsights]):
            The list of YouTube Channels
    """

    creator_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    creator_channels: MutableSequence["YouTubeChannelInsights"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="YouTubeChannelInsights",
        )
    )


class YouTubeMetrics(proto.Message):
    r"""YouTube Channel metrics.

    Attributes:
        subscriber_count (int):
            The number of subscribers.
        views_count (int):
            The total number of views.
        video_count (int):
            The total number of videos.
        likes_count (int):
            The total number of likes across all videos
            of this channel.
        shares_count (int):
            The total number of shares across all videos
            of this channel.
        comments_count (int):
            The total number of comments across all
            videos of this channel.
        engagement_rate (float):
            The lifetime engagement rate of this channel.
            The value is computed as the total number of
            likes, shares, and comments across all videos
            divided by the total number of videos.
        average_views_per_video (float):
            The average number of views per video in the
            last 28 days.
        average_likes_per_video (float):
            The average number of likes per video in the
            last 28 days.
        average_shares_per_video (float):
            The average number of shares per video in the
            last 28 days.
        average_comments_per_video (float):
            The average number of comments per video in
            the last 28 days.
        shorts_views_count (int):
            The total number of views across all Shorts
            videos of this channel.
        shorts_video_count (int):
            The total number of Shorts videos.
        is_active_shorts_creator (bool):
            When true, this channel has published a
            Shorts video in the last 90 days.
        is_brand_connect_creator (bool):
            When true, this creator can be partnered with
            to create original branded content using the
            Google Ads creator partnership platform,
            BrandConnect.

            See
            https://support.google.com/google-ads/answer/13828964
            for more information about BrandConnect.
    """

    subscriber_count: int = proto.Field(
        proto.INT64,
        number=1,
    )
    views_count: int = proto.Field(
        proto.INT64,
        number=2,
    )
    video_count: int = proto.Field(
        proto.INT64,
        number=3,
    )
    likes_count: int = proto.Field(
        proto.INT64,
        number=5,
    )
    shares_count: int = proto.Field(
        proto.INT64,
        number=6,
    )
    comments_count: int = proto.Field(
        proto.INT64,
        number=7,
    )
    engagement_rate: float = proto.Field(
        proto.DOUBLE,
        number=8,
    )
    average_views_per_video: float = proto.Field(
        proto.DOUBLE,
        number=9,
    )
    average_likes_per_video: float = proto.Field(
        proto.DOUBLE,
        number=10,
    )
    average_shares_per_video: float = proto.Field(
        proto.DOUBLE,
        number=11,
    )
    average_comments_per_video: float = proto.Field(
        proto.DOUBLE,
        number=12,
    )
    shorts_views_count: int = proto.Field(
        proto.INT64,
        number=13,
    )
    shorts_video_count: int = proto.Field(
        proto.INT64,
        number=14,
    )
    is_active_shorts_creator: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    is_brand_connect_creator: bool = proto.Field(
        proto.BOOL,
        number=15,
    )


class YouTubeChannelInsights(proto.Message):
    r"""YouTube Channel insights, and its metadata (such as channel
    name and channel ID), returned for a creator insights response.

    Attributes:
        display_name (str):
            The name of the YouTube Channel.
        youtube_channel (google.ads.googleads.v21.common.types.YouTubeChannelInfo):
            The YouTube Channel ID.
        channel_url (str):
            URL for the channel in the form of
            https://www.youtube.com/channel/{channel_id}.
        channel_description (str):
            Description of the channel.
        handle (str):
            The unique, short, and user-visible
            identifier for the channel starting with an "@"
            symbol (such as "@youtubecreators"). See
            https://support.google.com/youtube/answer/11585688
            for more information.
        thumbnail_url (str):
            URL for a 240px by 240px thumbnail image of
            the channel.
        publish_date (str):
            The date that the channel was created.
            Formatted as "yyyy-mm-dd".
        country_location (google.ads.googleads.v21.common.types.LocationInfo):
            The country with which the channel is
            associated.
        channel_metrics (google.ads.googleads.v21.services.types.YouTubeMetrics):
            The metrics for a YouTube Channel.
        channel_audience_attributes (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsAttributeMetadata]):
            The types of audiences and demographics
            linked to the channel's main audience. Audiences
            and demographics have a breakdown of subscriber
            share across dimensions of the same value, such
            as Age Range, Gender, and User Interest.
        channel_attributes (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsAttributeMetadata]):
            The attributes associated with the content
            made by a channel.
        top_videos (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsAttributeMetadata]):
            The top 10 videos for the channel.
        channel_type (str):
            Metadata string associated with the type of
            channel.
    """

    display_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    youtube_channel: criteria.YouTubeChannelInfo = proto.Field(
        proto.MESSAGE,
        number=2,
        message=criteria.YouTubeChannelInfo,
    )
    channel_url: str = proto.Field(
        proto.STRING,
        number=9,
    )
    channel_description: str = proto.Field(
        proto.STRING,
        number=10,
    )
    handle: str = proto.Field(
        proto.STRING,
        number=11,
    )
    thumbnail_url: str = proto.Field(
        proto.STRING,
        number=12,
    )
    publish_date: str = proto.Field(
        proto.STRING,
        number=13,
    )
    country_location: criteria.LocationInfo = proto.Field(
        proto.MESSAGE,
        number=14,
        message=criteria.LocationInfo,
    )
    channel_metrics: "YouTubeMetrics" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="YouTubeMetrics",
    )
    channel_audience_attributes: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    channel_attributes: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    top_videos: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=8,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    channel_type: str = proto.Field(
        proto.STRING,
        number=6,
    )


class SearchAudience(proto.Message):
    r"""A collection of audience attributes that describe an audience
    of viewers. This is used to search for topics trending for the
    defined audience.

    Attributes:
        audience_attributes (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsAttribute]):
            Required. Audience attributes that describe
            an audience of viewers. This is used to search
            for topics trending for the defined audience.
    """

    audience_attributes: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttribute
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=audience_insights_attribute.AudienceInsightsAttribute,
    )


class SearchTopics(proto.Message):
    r"""A collection of content topics to return trend information
    for.

    Attributes:
        entities (MutableSequence[google.ads.googleads.v21.common.types.AudienceInsightsEntity]):
            Required. A list of knowledge graph entities to retrieve
            trend information for. Supported entities are tagged with
            [InsightsKnowledgeGraphEntityCapabilities.CONTENT_TRENDING_INSIGHTS][].
    """

    entities: MutableSequence[
        audience_insights_attribute.AudienceInsightsEntity
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=audience_insights_attribute.AudienceInsightsEntity,
    )


class TrendInsight(proto.Message):
    r"""A trend insight for a given attribute.

    Attributes:
        trend_attribute (google.ads.googleads.v21.common.types.AudienceInsightsAttributeMetadata):
            The attribute this trend is for.
        trend_metrics (google.ads.googleads.v21.services.types.TrendInsightMetrics):
            Metrics associated with this trend.
        trend (google.ads.googleads.v21.enums.types.InsightsTrendEnum.InsightsTrend):
            The direction of trend (such as RISING or
            DECLINING).
    """

    trend_attribute: (
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ) = proto.Field(
        proto.MESSAGE,
        number=1,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    trend_metrics: "TrendInsightMetrics" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="TrendInsightMetrics",
    )
    trend: insights_trend.InsightsTrendEnum.InsightsTrend = proto.Field(
        proto.ENUM,
        number=3,
        enum=insights_trend.InsightsTrendEnum.InsightsTrend,
    )


class TrendInsightMetrics(proto.Message):
    r"""Metrics associated with a trend insight.

    Attributes:
        views_count (int):
            The number of views for this trend. This is
            only populated for SearchTopics requests.
        audience_share (float):
            The fraction (from 0 to 1 inclusive) of the
            requested audience that has has searched or
            viewed this trend. This is only populated for
            SearchAudience requests.
    """

    views_count: int = proto.Field(
        proto.INT64,
        number=1,
    )
    audience_share: float = proto.Field(
        proto.DOUBLE,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "MutateErrorEnum",
    },
)


class MutateErrorEnum(proto.Message):
    r"""Container for enum describing possible mutate errors."""

    class MutateError(proto.Enum):
        r"""Enum describing possible mutate errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            RESOURCE_NOT_FOUND (3):
                Requested resource was not found.
            ID_EXISTS_IN_MULTIPLE_MUTATES (7):
                Cannot mutate the same resource twice in one
                request.
            INCONSISTENT_FIELD_VALUES (8):
                The field's contents don't match another
                field that represents the same data.
            MUTATE_NOT_ALLOWED (9):
                Mutates are not allowed for the requested
                resource.
            RESOURCE_NOT_IN_GOOGLE_ADS (10):
                The resource isn't in Google Ads. It belongs
                to another ads system.
            RESOURCE_ALREADY_EXISTS (11):
                The resource being created already exists.
            RESOURCE_DOES_NOT_SUPPORT_VALIDATE_ONLY (12):
                This resource cannot be used with "validate_only".
            OPERATION_DOES_NOT_SUPPORT_PARTIAL_FAILURE (16):
                This operation cannot be used with "partial_failure".
            RESOURCE_READ_ONLY (13):
                Attempt to write to read-only fields.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        RESOURCE_NOT_FOUND = 3
        ID_EXISTS_IN_MULTIPLE_MUTATES = 7
        INCONSISTENT_FIELD_VALUES = 8
        MUTATE_NOT_ALLOWED = 9
        RESOURCE_NOT_IN_GOOGLE_ADS = 10
        RESOURCE_ALREADY_EXISTS = 11
        RESOURCE_DOES_NOT_SUPPORT_VALIDATE_ONLY = 12
        OPERATION_DOES_NOT_SUPPORT_PARTIAL_FAILURE = 16
        RESOURCE_READ_ONLY = 13


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import tracking_code_page_format
from google.ads.googleads.v20.enums.types import tracking_code_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.common",
    marshal="google.ads.googleads.v20",
    manifest={
        "TagSnippet",
    },
)


class TagSnippet(proto.Message):
    r"""The site tag and event snippet pair for a TrackingCodeType.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        type_ (google.ads.googleads.v20.enums.types.TrackingCodeTypeEnum.TrackingCodeType):
            The type of the generated tag snippets for
            tracking conversions.
        page_format (google.ads.googleads.v20.enums.types.TrackingCodePageFormatEnum.TrackingCodePageFormat):
            The format of the web page where the tracking
            tag and snippet will be installed, for example,
            HTML.
        global_site_tag (str):
            The site tag that adds visitors to your basic
            remarketing lists and sets new cookies on your
            domain.

            This field is a member of `oneof`_ ``_global_site_tag``.
        event_snippet (str):
            The event snippet that works with the site
            tag to track actions that should be counted as
            conversions.

            This field is a member of `oneof`_ ``_event_snippet``.
    """

    type_: tracking_code_type.TrackingCodeTypeEnum.TrackingCodeType = (
        proto.Field(
            proto.ENUM,
            number=1,
            enum=tracking_code_type.TrackingCodeTypeEnum.TrackingCodeType,
        )
    )
    page_format: (
        tracking_code_page_format.TrackingCodePageFormatEnum.TrackingCodePageFormat
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=tracking_code_page_format.TrackingCodePageFormatEnum.TrackingCodePageFormat,
    )
    global_site_tag: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    event_snippet: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Demonstrates various operations involved in remarketing.

Operations include (a) creating a user list based on visitors to a website,
(b) targeting a user list with an ad group criterion, (c) updating the bid
modifier on an ad group criterion, (d) finding and removing all ad group
criteria under a given campaign, (e) targeting a user list with a campaign
criterion, and (f) updating the bid modifier on a campaign criterion. It is
unlikely that users will need to perform all of these operations consecutively,
and all of the operations contained herein are meant of for illustrative
purposes.
"""


import argparse
import sys
from typing import List
from uuid import uuid4

from google.api_core import protobuf_helpers

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.user_lists import (
    FlexibleRuleOperandInfo,
    FlexibleRuleUserListInfo,
    UserListRuleItemGroupInfo,
    UserListRuleItemInfo,
)
from google.ads.googleads.v22.resources.types.ad_group_criterion import (
    AdGroupCriterion,
)
from google.ads.googleads.v22.resources.types.campaign_criterion import (
    CampaignCriterion,
)
from google.ads.googleads.v22.resources.types.user_list import UserList
from google.ads.googleads.v22.services.services.ad_group_criterion_service import (
    AdGroupCriterionServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_criterion_service import (
    CampaignCriterionServiceClient,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.services.user_list_service import (
    UserListServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_criterion_service import (
    AdGroupCriterionOperation,
    MutateAdGroupCriteriaResponse,
)
from google.ads.googleads.v22.services.types.campaign_criterion_service import (
    CampaignCriterionOperation,
    MutateCampaignCriteriaResponse,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    GoogleAdsRow,
    SearchGoogleAdsRequest,
    SearchGoogleAdsResponse,
)
from google.ads.googleads.v22.services.types.user_list_service import (
    MutateUserListsResponse,
    UserListOperation,
)


def main(
    client: GoogleAdsClient,
    customer_id: str,
    campaign_id: str,
    ad_group_id: str,
    bid_modifier_value: float,
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a str client customer ID used to create a user list and
            other various entities required for the example.
        campaign_id: a str ID for a campaign used to create an ad group
            criterion that targets members of a user list.
        ad_group_id: a str ID for an ad group used to create an ad group
            criterion that targets members of a user list.
        bid_modifier_value: a float that specifies a modifier on the bid amount
            for newly created ad group criterion.
    """
    user_list_resource_name: str = create_user_list(client, customer_id)
    ad_group_criterion_resource_name: str = target_ads_in_ad_group_to_user_list(
        client, customer_id, ad_group_id, user_list_resource_name
    )
    modify_ad_group_bids(
        client,
        customer_id,
        ad_group_criterion_resource_name,
        bid_modifier_value,
    )
    remove_existing_criteria_from_ad_group(client, customer_id, campaign_id)
    campaign_criterion_resource_name: str = target_ads_in_campaign_to_user_list(
        client, customer_id, campaign_id, user_list_resource_name
    )
    modify_campaign_bids(
        client,
        customer_id,
        campaign_criterion_resource_name,
        bid_modifier_value,
    )


# [START setup_remarketing]
def create_user_list(client: GoogleAdsClient, customer_id: str) -> str:
    """Creates a user list targeting users that have visited a given URL.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a str client customer ID used to create a user list.

    Returns:
        a str resource name for the newly created user list.
    """
    # Creates a UserListOperation.
    user_list_operation: UserListOperation = client.get_type(
        "UserListOperation"
    )
    # Creates a UserList.
    user_list: UserList = user_list_operation.create
    user_list.name = f"All visitors to example.com #{uuid4()}"
    user_list.description = "Any visitor to any page of example.com"
    user_list.membership_status = client.enums.UserListMembershipStatusEnum.OPEN
    user_list.membership_life_span = 365
    # Optional: To include past users in the user list, set the
    # prepopulation_status to REQUESTED.
    user_list.rule_based_user_list.prepopulation_status = (
        client.enums.UserListPrepopulationStatusEnum.REQUESTED
    )
    # Specifies that the user list targets visitors of a page with a URL that
    # contains 'example.com'.
    user_list_rule_item_group_info: UserListRuleItemGroupInfo = client.get_type(
        "UserListRuleItemGroupInfo"
    )
    user_list_rule_item_info: UserListRuleItemInfo = client.get_type(
        "UserListRuleItemInfo"
    )
    # Uses a built-in parameter to create a domain URL rule.
    user_list_rule_item_info.name = "url__"
    user_list_rule_item_info.string_rule_item.operator = (
        client.enums.UserListStringRuleItemOperatorEnum.CONTAINS
    )
    user_list_rule_item_info.string_rule_item.value = "example.com"
    user_list_rule_item_group_info.rule_items.append(user_list_rule_item_info)

    # Specify that the user list targets visitors of a page based on the
    # provided rule.
    flexible_rule_user_list_info: FlexibleRuleUserListInfo = (
        user_list.rule_based_user_list.flexible_rule_user_list
    )
    flexible_rule_user_list_info.inclusive_rule_operator = (
        client.enums.UserListFlexibleRuleOperatorEnum.AND
    )
    # Inclusive operands are joined together with the specified
    # inclusive rule operator.
    rule_operand: FlexibleRuleOperandInfo = client.get_type(
        "FlexibleRuleOperandInfo"
    )
    rule_operand.rule.rule_item_groups.extend([user_list_rule_item_group_info])
    rule_operand.lookback_window_days = 7
    flexible_rule_user_list_info.inclusive_operands.append(rule_operand)

    user_list_service: UserListServiceClient = client.get_service(
        "UserListService"
    )
    response: MutateUserListsResponse = user_list_service.mutate_user_lists(
        customer_id=customer_id, operations=[user_list_operation]
    )
    resource_name: str = response.results[0].resource_name
    print(f"Created user list with resource name: '{resource_name}'")
    return resource_name
    # [END setup_remarketing]


# [START setup_remarketing_1]
def target_ads_in_ad_group_to_user_list(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    user_list_resource_name: str,
) -> str:
    """Creates an ad group criterion that targets a user list with an ad group.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a str client customer ID used to create an ad group
            criterion.
        ad_group_id: a str ID for an ad group used to create an ad group
            criterion that targets members of a user list.
        user_list_resource_name: a str resource name for a user list.

    Returns:
        a str resource name for an ad group criterion.
    """
    ad_group_criterion_operation: AdGroupCriterionOperation = client.get_type(
        "AdGroupCriterionOperation"
    )
    # Creates the ad group criterion targeting members of the user list.
    ad_group_criterion: AdGroupCriterion = ad_group_criterion_operation.create
    ad_group_criterion.ad_group = client.get_service(
        "AdGroupService"
    ).ad_group_path(customer_id, ad_group_id)
    ad_group_criterion.user_list.user_list = user_list_resource_name

    ad_group_criterion_service: AdGroupCriterionServiceClient = (
        client.get_service("AdGroupCriterionService")
    )
    response: MutateAdGroupCriteriaResponse = (
        ad_group_criterion_service.mutate_ad_group_criteria(
            customer_id=customer_id, operations=[ad_group_criterion_operation]
        )
    )
    resource_name: str = response.results[0].resource_name
    print(
        "Successfully created ad group criterion with resource name: "
        f"'{resource_name}' targeting user list with resource name: "
        f"'{user_list_resource_name}' and with ad group with ID "
        f"{ad_group_id}."
    )
    return resource_name
    # [END setup_remarketing_1]


def modify_ad_group_bids(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_criterion_resource_name: str,
    bid_modifier_value: float,
) -> None:
    """Updates the bid modifier on an ad group criterion.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a str client customer ID.
        ad_group_criterion.resource_name: a str resource name for an ad group
            criterion.
        bid_modifier_value: a float value specifying an ad group criterion
            bid modifier.
    """
    # Constructs an operation that will update the ad group criterion with the
    # specified resource name.
    ad_group_criterion_operation: AdGroupCriterionOperation = client.get_type(
        "AdGroupCriterionOperation"
    )
    ad_group_criterion: AdGroupCriterion = ad_group_criterion_operation.update
    # Creates the ad group criterion with a bid modifier. You may alternatively
    # set the bid for the ad group criterion directly.
    ad_group_criterion.resource_name = ad_group_criterion_resource_name
    ad_group_criterion.bid_modifier = bid_modifier_value
    # Using the FieldMasks utility to derive the update mask tells the Google
    # Ads API which attributes of the ad group criterion you want to change.
    client.copy_from(
        ad_group_criterion_operation.update_mask,
        protobuf_helpers.field_mask(None, ad_group_criterion._pb),
    )
    ad_group_criterion_service: AdGroupCriterionServiceClient = (
        client.get_service("AdGroupCriterionService")
    )
    response: MutateAdGroupCriteriaResponse = (
        ad_group_criterion_service.mutate_ad_group_criteria(
            customer_id=customer_id, operations=[ad_group_criterion_operation]
        )
    )
    print(
        "Updated bid for ad group criterion with resource name: "
        f"'{response.results[0].resource_name}'"
    )


# [START setup_remarketing_3]
def remove_existing_criteria_from_ad_group(
    client: GoogleAdsClient, customer_id: str, campaign_id: str
) -> None:
    """Removes all ad group criteria targeting a user list under a campaign.

    This is a necessary step before targeting a user list at the campaign level.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a str client customer ID.
        campaign_id: a str ID for a campaign that will have all ad group
            criteria that targets user lists removed.
    """
    # Retrieves all of the ad group criteria under a campaign.
    all_ad_group_criteria: List[str] = get_user_list_ad_group_criteria(
        client, customer_id, campaign_id
    )

    # Creates a list of remove operations.
    remove_operations: List[AdGroupCriterionOperation] = []
    for ad_group_criterion_resource_name in all_ad_group_criteria:
        remove_operation: AdGroupCriterionOperation = client.get_type(
            "AdGroupCriterionOperation"
        )
        remove_operation.remove = ad_group_criterion_resource_name
        remove_operations.append(remove_operation)

    ad_group_criterion_service: AdGroupCriterionServiceClient = (
        client.get_service("AdGroupCriterionService")
    )
    response: MutateAdGroupCriteriaResponse = (
        ad_group_criterion_service.mutate_ad_group_criteria(
            customer_id=customer_id, operations=remove_operations
        )
    )
    print(
        "Successfully removed ad group criterion with resource name: "
        f"'{response.results[0].resource_name}'"
    )
    # [END setup_remarketing_3]


# [START setup_remarketing_2]
def get_user_list_ad_group_criteria(
    client: GoogleAdsClient, customer_id: str, campaign_id: str
) -> List[str]:
    """Finds all of user list ad group criteria under a campaign.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a str client customer ID.
        campaign_id: a str campaign ID.

    Returns:
        a list of ad group criterion resource names.
    """
    # Creates a query that retrieves all of the ad group criteria under a
    # campaign.
    query: str = f"""
        SELECT
          ad_group_criterion.criterion_id
        FROM ad_group_criterion
        WHERE campaign.id = {campaign_id}
        AND ad_group_criterion.type = USER_LIST"""

    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    search_request: SearchGoogleAdsRequest = client.get_type(
        "SearchGoogleAdsRequest"
    )
    search_request.customer_id = customer_id
    search_request.query = query
    response: SearchGoogleAdsResponse = googleads_service.search(
        request=search_request
    )

    # Iterates over all rows in all pages. Prints the user list criteria and
    # adds the ad group criteria resource names to the list.
    user_list_criteria: List[str] = []
    row: GoogleAdsRow
    for row in response:
        resource_name: str = row.ad_group_criterion.resource_name
        print(
            "Ad group criterion with resource name '{resource_name}' was "
            "found."
        )
        user_list_criteria.append(resource_name)

    return user_list_criteria
    # [END setup_remarketing_2]


# [START setup_remarketing_4]
def target_ads_in_campaign_to_user_list(
    client: GoogleAdsClient,
    customer_id: str,
    campaign_id: str,
    user_list_resource_name: str,
) -> str:
    """Creates a campaign criterion that targets a user list with a campaign.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a str client customer ID used to create an campaign
            criterion.
        campaign_id: a str ID for a campaign used to create a campaign
            criterion that targets members of a user list.
        user_list_resource_name: a str resource name for a user list.

    Returns:
        a str resource name for a campaign criterion.
    """
    campaign_criterion_operation: CampaignCriterionOperation = client.get_type(
        "CampaignCriterionOperation"
    )
    campaign_criterion: CampaignCriterion = campaign_criterion_operation.create
    campaign_criterion.campaign = client.get_service(
        "CampaignService"
    ).campaign_path(customer_id, campaign_id)
    campaign_criterion.user_list.user_list = user_list_resource_name

    campaign_criterion_service: CampaignCriterionServiceClient = (
        client.get_service("CampaignCriterionService")
    )
    response: MutateCampaignCriteriaResponse = (
        campaign_criterion_service.mutate_campaign_criteria(
            customer_id=customer_id, operations=[campaign_criterion_operation]
        )
    )
    resource_name: str = response.results[0].resource_name
    print(
        "Successfully created campaign criterion with resource name "
        f"'{resource_name}' targeting user list with resource name "
        f"'{user_list_resource_name}' with campaign with ID {campaign_id}"
    )
    return resource_name
    # [END setup_remarketing_4]


def modify_campaign_bids(
    client: GoogleAdsClient,
    customer_id: str,
    campaign_criterion_resource_name: str,
    bid_modifier_value: float,
) -> None:
    """Updates the bid modifier on a campaign criterion.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a str client customer ID.
        campaign_criterion_resource_name: a str resource name for a campaign
            criterion.
        bid_modifier_value: a float value specifying a campaign criterion
            bid modifier.
    """
    # Constructs an operation that will update the campaign criterion with the
    # specified resource name.
    campaign_criterion_operation: CampaignCriterionOperation = client.get_type(
        "CampaignCriterionOperation"
    )
    campaign_criterion: CampaignCriterion = campaign_criterion_operation.update
    campaign_criterion.resource_name = campaign_criterion_resource_name
    campaign_criterion.bid_modifier = bid_modifier_value

    # Using the FieldMasks utility to derive the update mask tells the Google
    # Ads API which attributes of the campaign criterion you want to change.
    client.copy_from(
        campaign_criterion_operation.update_mask,
        protobuf_helpers.field_mask(None, campaign_criterion._pb),
    )

    campaign_criterion_service: CampaignCriterionServiceClient = (
        client.get_service("CampaignCriterionService")
    )
    response: MutateCampaignCriteriaResponse = (
        campaign_criterion_service.mutate_campaign_criteria(
            customer_id=customer_id, operations=[campaign_criterion_operation]
        )
    )
    print(
        "Successfully updated the bid for campaign criterion with resource "
        f"name: '{response.results[0].resource_name}'"
    )


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Demonstrates various operations involved in remarketing."
    )
    # The following arguments are required to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help=(
            "A Google Ads customer ID used to create a user list and other "
            "various entities required for the example."
        ),
    )
    parser.add_argument(
        "-i",
        "--campaign_id",
        type=str,
        required=True,
        help=(
            "The ID for a campaign that will have its ad group criteria "
            "modified to target user lists members."
        ),
    )
    parser.add_argument(
        "-a",
        "--ad_group_id",
        type=str,
        required=True,
        help=(
            "The ID for an ad group used to create an ad group criterion "
            "that targets members of a user list."
        ),
    )
    # The following argument is optional.
    parser.add_argument(
        "-b",
        "--bid_modifier_value",
        type=float,
        default=1.5,
        help=(
            "A float that specifies a modifier on the bid amount "
            "for newly created ad group criterion."
        ),
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.campaign_id,
            args.ad_group_id,
            args.bid_modifier_value,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "GenderView",
    },
)


class GenderView(proto.Message):
    r"""A gender view. The gender_view resource reflects the effective
    serving state, rather than what criteria were added. An ad group
    without gender criteria by default shows to all genders, so all
    genders appear in gender_view with stats.

    Attributes:
        resource_name (str):
            Output only. The resource name of the gender view. Gender
            view resource names have the form:

            ``customers/{customer_id}/genderViews/{ad_group_id}~{criterion_id}``
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Adds a shopping listing scope to a shopping campaign.

The example will construct and add a new listing scope which will act as the
inventory filter for the campaign. The campaign will only advertise products
that match the following requirements:

* Brand is "google".
* Custom label 0 is "top_selling_products".
* Product type (level 1) is "electronics".
* Product type (level 2) is "smartphones".

Only one listing scope is allowed per campaign. Remove any existing listing
scopes before running this example.
"""

import argparse
import sys
from typing import List

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.criteria import (
    ListingDimensionInfo,
)
from google.ads.googleads.v22.enums.types.product_custom_attribute_index import (
    ProductCustomAttributeIndexEnum,
)
from google.ads.googleads.v22.enums.types.product_type_level import (
    ProductTypeLevelEnum,
)
from google.ads.googleads.v22.resources.types.campaign_criterion import (
    CampaignCriterion,
)
from google.ads.googleads.v22.services.services.campaign_criterion_service import (
    CampaignCriterionServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_service import (
    CampaignServiceClient,
)
from google.ads.googleads.v22.services.types.campaign_criterion_service import (
    CampaignCriterionOperation,
    MutateCampaignCriteriaResponse,
)


def main(client: GoogleAdsClient, customer_id: str, campaign_id: str) -> None:
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )
    campaign_criterion_operation: CampaignCriterionOperation = client.get_type(
        "CampaignCriterionOperation"
    )
    campaign_criterion: CampaignCriterion = campaign_criterion_operation.create
    campaign_criterion.campaign = campaign_service.campaign_path(
        customer_id, campaign_id
    )

    # A listing scope allows you to filter the products that will be included in
    # a given campaign. You can specify multiple dimensions with conditions that
    # must be met for a product to be included in a campaign.
    # A typical listing scope might only have a few dimensions. This example
    # demonstrates a range of different dimensions you could use.
    dimensions: List[ListingDimensionInfo] = (
        campaign_criterion.listing_scope.dimensions
    )

    product_brand_dimension: ListingDimensionInfo = client.get_type(
        "ListingDimensionInfo"
    )
    product_brand_dimension.product_brand.value = "google"
    dimensions.append(product_brand_dimension)

    product_custom_attribute_index_enum: ProductCustomAttributeIndexEnum = (
        client.enums.ProductCustomAttributeIndexEnum
    )
    product_custom_attribute_dimension: ListingDimensionInfo = client.get_type(
        "ListingDimensionInfo"
    )
    product_custom_attribute = (
        product_custom_attribute_dimension.product_custom_attribute
    )
    product_custom_attribute.index = product_custom_attribute_index_enum.INDEX0
    product_custom_attribute.value = "top_selling_products"
    dimensions.append(product_custom_attribute_dimension)

    product_type_level_enum: ProductTypeLevelEnum = (
        client.enums.ProductTypeLevelEnum
    )
    product_type_dimension_1: ListingDimensionInfo = client.get_type(
        "ListingDimensionInfo"
    )
    product_type = product_type_dimension_1.product_type
    product_type.level = product_type_level_enum.LEVEL1
    product_type.value = "electronics"
    dimensions.append(product_type_dimension_1)

    product_type_dimension_2: ListingDimensionInfo = client.get_type(
        "ListingDimensionInfo"
    )
    product_type = product_type_dimension_2.product_type
    product_type.level = product_type_level_enum.LEVEL2
    product_type.value = "smartphones"
    dimensions.append(product_type_dimension_2)

    campaign_criterion_service: CampaignCriterionServiceClient = (
        client.get_service("CampaignCriterionService")
    )

    campaign_criterion_response: MutateCampaignCriteriaResponse = (
        campaign_criterion_service.mutate_campaign_criteria(
            customer_id=customer_id, operations=[campaign_criterion_operation]
        )
    )

    print(
        f"Added {len(campaign_criterion_response.results)} campaign "
        "criteria:"
    )
    for criterion in campaign_criterion_response.results:
        print(criterion.resource_name)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=("Adds a shopping listing scope to a shopping campaign.")
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-i", "--campaign_id", type=str, required=True, help="The campaign ID."
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.campaign_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status'
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "AudienceErrorEnum",
    },
)


class AudienceErrorEnum(proto.Message):
    r"""Container for enum describing possible audience errors."""

    class AudienceError(proto.Enum):
        r"""Enum describing possible audience errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            NAME_ALREADY_IN_USE (2):
                An audience with this name already exists.
            DIMENSION_INVALID (3):
                A dimension within the audience definition is
                not valid.
            AUDIENCE_SEGMENT_NOT_FOUND (4):
                One of the audience segment added is not
                found.
            AUDIENCE_SEGMENT_TYPE_NOT_SUPPORTED (5):
                One of the audience segment type is not
                supported.
            DUPLICATE_AUDIENCE_SEGMENT (6):
                The same segment already exists in this
                audience.
            TOO_MANY_SEGMENTS (7):
                Audience can't have more than allowed number
                segments.
            TOO_MANY_DIMENSIONS_OF_SAME_TYPE (8):
                Audience can't have multiple dimensions of
                same type.
            IN_USE (9):
                The audience cannot be removed, because it is
                currently used in an ad group criterion or asset
                group signal in an (enabled or paused) ad group
                or campaign.
            MISSING_ASSET_GROUP_ID (10):
                Asset Group scoped audience requires an asset
                group ID.
            CANNOT_CHANGE_FROM_CUSTOMER_TO_ASSET_GROUP_SCOPE (11):
                Audience scope may not be changed from
                Customer to AssetGroup.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NAME_ALREADY_IN_USE = 2
        DIMENSION_INVALID = 3
        AUDIENCE_SEGMENT_NOT_FOUND = 4
        AUDIENCE_SEGMENT_TYPE_NOT_SUPPORTED = 5
        DUPLICATE_AUDIENCE_SEGMENT = 6
        TOO_MANY_SEGMENTS = 7
        TOO_MANY_DIMENSIONS_OF_SAME_TYPE = 8
        IN_USE = 9
        MISSING_ASSET_GROUP_ID = 10
        CANNOT_CHANGE_FROM_CUSTOMER_TO_ASSET_GROUP_SCOPE = 11


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "AudienceInsightsDimensionEnum",
    },
)


class AudienceInsightsDimensionEnum(proto.Message):
    r"""Container for enum describing insights dimensions."""

    class AudienceInsightsDimension(proto.Enum):
        r"""Possible dimensions for use in generating insights.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            CATEGORY (2):
                A Product & Service category.
            KNOWLEDGE_GRAPH (3):
                A Knowledge Graph entity.
            GEO_TARGET_COUNTRY (4):
                A country, represented by a geo target.
            SUB_COUNTRY_LOCATION (5):
                A geographic location within a country.
            YOUTUBE_CHANNEL (6):
                A YouTube channel.
            AFFINITY_USER_INTEREST (8):
                An Affinity UserInterest.
            IN_MARKET_USER_INTEREST (9):
                An In-Market UserInterest.
            PARENTAL_STATUS (10):
                A Parental Status value (parent, or not a
                parent).
            INCOME_RANGE (11):
                A household income percentile range.
            AGE_RANGE (12):
                An age range.
            GENDER (13):
                A gender.
            YOUTUBE_VIDEO (14):
                A YouTube video.
            DEVICE (15):
                A device type, such as Mobile, Desktop,
                Tablet, and Connected TV.
            YOUTUBE_LINEUP (16):
                A YouTube Lineup.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CATEGORY = 2
        KNOWLEDGE_GRAPH = 3
        GEO_TARGET_COUNTRY = 4
        SUB_COUNTRY_LOCATION = 5
        YOUTUBE_CHANNEL = 6
        AFFINITY_USER_INTEREST = 8
        IN_MARKET_USER_INTEREST = 9
        PARENTAL_STATUS = 10
        INCOME_RANGE = 11
        AGE_RANGE = 12
        GENDER = 13
        YOUTUBE_VIDEO = 14
        DEVICE = 15
        YOUTUBE_LINEUP = 16


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "FeedItemTargetErrorEnum",
    },
)


class FeedItemTargetErrorEnum(proto.Message):
    r"""Container for enum describing possible feed item target
    errors.

    """

    class FeedItemTargetError(proto.Enum):
        r"""Enum describing possible feed item target errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            MUST_SET_TARGET_ONEOF_ON_CREATE (2):
                On CREATE, the FeedItemTarget must have a
                populated field in the oneof target.
            FEED_ITEM_TARGET_ALREADY_EXISTS (3):
                The specified feed item target already
                exists, so it cannot be added.
            FEED_ITEM_SCHEDULES_CANNOT_OVERLAP (4):
                The schedules for a given feed item cannot
                overlap.
            TARGET_LIMIT_EXCEEDED_FOR_GIVEN_TYPE (5):
                Too many targets of a given type were added
                for a single feed item.
            TOO_MANY_SCHEDULES_PER_DAY (6):
                Too many AdSchedules are enabled for the feed
                item for the given day.
            CANNOT_HAVE_ENABLED_CAMPAIGN_AND_ENABLED_AD_GROUP_TARGETS (7):
                A feed item may either have an enabled
                campaign target or an enabled ad group target.
            DUPLICATE_AD_SCHEDULE (8):
                Duplicate ad schedules aren't allowed.
            DUPLICATE_KEYWORD (9):
                Duplicate keywords aren't allowed.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        MUST_SET_TARGET_ONEOF_ON_CREATE = 2
        FEED_ITEM_TARGET_ALREADY_EXISTS = 3
        FEED_ITEM_SCHEDULES_CANNOT_OVERLAP = 4
        TARGET_LIMIT_EXCEEDED_FOR_GIVEN_TYPE = 5
        TOO_MANY_SCHEDULES_PER_DAY = 6
        CANNOT_HAVE_ENABLED_CAMPAIGN_AND_ENABLED_AD_GROUP_TARGETS = 7
        DUPLICATE_AD_SCHEDULE = 8
        DUPLICATE_KEYWORD = 9


__all__ = tuple(sorted(__protobuf__.manifest))
